from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2013_12.DataManagement import GetStyleSheetDatasetInfo, GetStyleSheetClassificationData
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Internal.AWS2._2016_03.DataManagement import StylesheetContext
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetStyleSheetOutput(TcBaseObj):
    """
    Output from getStyleSheet operation.
    
    :var processedPage: The name of the page that was processed. If the operation was invoked with processEntireXRT set
    to true, the this field is empty.  If the operation was invoked with processEntireXRT set to false, this this field
    is populated with the page that was processed.  Typically the value of the processedPage would be the same as the
    targetPage, except when the targetPage value is empty or not found in the stylesheet.  In that case, then the first
    page in the stylesheet is processed.
    :var datasetInfo: The information about the stylesheet.
    :var context: The context to which this stylesheet was registered
    :var imageTypeTicketMap: Map (string/string) where the key is the image resolution type from the stylesheet and
    value is the FMS ticket of the image for that resolution type.When resolution is not specified in the stylesheet or
    is invalid, FMS ticket mentioned in the default key will be populated.If the styesheet being rendered is SUMMARY,
    default value will be FMS ticket of the High resolution derived Dataset else default value would be FMS ticket of
    the Medium resolution derived Dataset.If derived Dataset does not exist,default value will be FMS ticket of the
    original Dataset.
    :var propDescriptors: List of LogicalPropertyDescriptors required to render the object properties in stylesheet.
    :var additionalObjects: List of additional objects to support Descriptor use cases in stylesheeet
    :var objectSetInfo: Structure containing the source field from the stylesheet, vector of LogicalObjects and list of
    LogicalPropertyDescriptors required to present inside that objectSet in the application.
    :var tablePropObjectMap: A map(string, list of POM_object) where key is the name of the table property and the
    value is a list of Table Row objects which represents the table row data for that property. 
    Note: This output data structure is not available for consumption in Teamcenter 10.1.x Release stream.
    :var localeMap: Map (string, string) where the key is the text, title, or titleKey string from the stylesheet and
    the value is the localized string.
    :var jtFileMap: Map (business object, string) where the key is the business object to render, and the value is the
    thumbnail ticket for the image to download.
    :var visiblePages: The visible pages in the XRT.  Each index in the array indicates whether that page can be
    displayed.
    :var classificationData: The classification data for the object to be rendered.
    :var objectToRender: The object to render in the UI.  For revise, this is the revise descriptor.  For all other
    operations it is the input businessObject.
    :var patternMap: Key is the property name, and value is the Pattern Information which are valid for that property
    """
    processedPage: str = ''
    datasetInfo: GetStyleSheetDatasetInfo = None
    context: StylesheetContext = None
    imageTypeTicketMap: StringMap7 = None
    propDescriptors: List[ViewModelPropertyDescriptor] = ()
    additionalObjects: AdditionalObjectsStruct = None
    objectSetInfo: List[ObjectSetInfo] = ()
    tablePropObjectMap: TablePropObjectMap3 = None
    localeMap: StringMap7 = None
    jtFileMap: ThumbnailMap6 = None
    visiblePages: List[bool] = ()
    classificationData: List[GetStyleSheetClassificationData] = ()
    objectToRender: ViewModelObject = None
    patternMap: PatternMap3 = None


@dataclass
class GetStyleSheetResponse(TcBaseObj):
    """
    Response sent to client from the getStyleSheet operation.
    
    :var output: The list of output information.  One for each input object
    :var serviceData: The SOA service data.
    """
    output: List[GetStyleSheetOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetViewModelPropertiesOutput(TcBaseObj):
    """
    Output structure containing ViewModelObject structures and corresponding ViewModelPropertyDescriptor structures.
    
    :var objects: A list of ViewModelObject structures.
    :var propDescriptors: A list of ViewModelPropertyDescriptors.
    """
    objects: List[ViewModelObject] = ()
    propDescriptors: List[ViewModelPropertyDescriptor] = ()


@dataclass
class GetViewModelPropsResponse(TcBaseObj):
    """
    Response for getViewModelProperties service operation
    
    :var output: Single GetViewModelPropertiesOutput structure.
    :var serviceData: Service Data.
    """
    output: GetViewModelPropertiesOutput = None
    serviceData: ServiceData = None


@dataclass
class GetViewerDataIn(TcBaseObj):
    """
    The input structure for GetViewerData.
    
    :var obj: The input object for which associated dataset and relevant viewer
    needs to be fetched.
    :var dataset: The dataset object which is currently displayed in the viewer. Sending dataset object as empty
    results in fetching the default viewer associated with the input object.
    :var direction: Indicates the direction for dataset retrieval from input object based on dataset currently
    displayed in the viewer. If direction is null or empty, then the dataset input argument is ignored and input object
    is processed to get the associated dataset and relevant viewer. If direction is 'next', then the input object is
    processed to find the dataset and relevant viewer that is next in line to the received dataset argument. If
    direction is 'previous', then the input object is processed to find the dataset and relevant viewer that is
    previous to the received dataset argument. Valid values for direction are "", next, previous.
    """
    obj: BusinessObject = None
    dataset: BusinessObject = None
    direction: str = ''


@dataclass
class GetViewerDataOutput(TcBaseObj):
    """
    Output structure for viewer data.
    
    :var dataset: The dataset whose associated file needs to be displayed in the viewer.
    :var views: The list of ViewData objects.
    :var hasMoreDatasets: Flag to indicate if more than one datasets are related to input object.
    """
    dataset: BusinessObject = None
    views: List[ViewData] = ()
    hasMoreDatasets: bool = False


@dataclass
class GetViewerDataResponse(TcBaseObj):
    """
    Response structure for getViewerData SOA operation.
    
    :var output: The list of GetViewerDataOutput objects.
    :var serviceData: The SOA service data.
    """
    output: GetViewerDataOutput = None
    serviceData: ServiceData = None


@dataclass
class AdditionalObjectsStruct(TcBaseObj):
    """
    Structure to store additional ViewModelObjects for descriptors in stylesheet
    
    :var additionalObjects: List of additional objects
    :var propDescriptors: ViewModelPropertyDescriptors for additional Objects
    """
    additionalObjects: List[ViewModelObject] = ()
    propDescriptors: List[ViewModelPropertyDescriptor] = ()


@dataclass
class LoadViewModelForEditingInfo(TcBaseObj):
    """
    Structure represents the parameters required to load the view model objects for editing.
    
    :var objs: Input objects
    :var propertyNames: Property names to edit
    :var isPessimisticLock: If true, the object needs to be checked out before making any edits (pessimistic locking).
    The locking of object is done during loadViewModelForEditing() call. Otherwise, the operation takes optimistic lock
    on the object. In optimistic locking the object is locked (based on object last set date "lsd" property) before
    performing the actual save and lock is released after save is complete.
    """
    objs: List[BusinessObject] = ()
    propertyNames: List[str] = ()
    isPessimisticLock: bool = False


@dataclass
class LoadViewModelForEditingOutput(TcBaseObj):
    """
    Output structure of loadViewModelForEditing operation
    
    :var viewModelObjects: View model objects corresponding to the input model object
    """
    viewModelObjects: List[ViewModelObject] = ()


@dataclass
class LoadViewModelForEditingResponse(TcBaseObj):
    """
    Response of LoadViewModelForEditing service operation
    
    :var outputs: outputs of loadViewModelForEditing operation
    :var serviceData: Partial errors are returned in the Service Data.
    """
    outputs: List[LoadViewModelForEditingOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ObjectSetColumnInfo(TcBaseObj):
    """
    Structure representing a sigle column in the object set table
    
    :var propDescriptor: Property Descriptor for the properties required for ViewModelObjects.
    :var userSortable: If true, the column is to be used for sorting.
    :var isRelationProperty: If true, the column represents a relation property; otherwise, other property types like
    attribute, reference, compound or runtime are used.
    """
    propDescriptor: ViewModelPropertyDescriptor = None
    userSortable: bool = False
    isRelationProperty: bool = False


@dataclass
class ObjectSetInfo(TcBaseObj):
    """
    Contains information about the object set source in the xml rendering stylesheet that is to be mapped to the
    Logical business objects
    
    :var xmlSource: Object Source in the xml
    :var rows: A list of ObjectSetRowInfo objects representing the rows in Objectset.
    :var columns: A list of ObjectSetColumnInfo objects representing the columns in Objectset.
    """
    xmlSource: str = ''
    rows: List[ObjectSetRowInfo] = ()
    columns: List[ObjectSetColumnInfo] = ()


@dataclass
class ObjectSetRowInfo(TcBaseObj):
    """
    Structure representing a single row in the objectset table.
    
    :var viewModelObject: ModelObject to be displayed in the row of the ObjectSet.
    :var relationTypeName: Relation Type Name
    :var relationTypeDisplayName: Relation Type display name
    :var relationObject: ImaRelation object
    :var relTypeObject: Model Object for ImanRelationType
    :var refPropertyName: Reference Property Name
    """
    viewModelObject: ViewModelObject = None
    relationTypeName: str = ''
    relationTypeDisplayName: str = ''
    relationObject: ViewModelObject = None
    relTypeObject: BusinessObject = None
    refPropertyName: str = ''


@dataclass
class PatternInfo3(TcBaseObj):
    """
    Structure defining pattern information
    
    :var patterns: The naming rule patterns
    :var preferredPattern: The preferred pattern to use. Typically would be the last used pattern.
    :var condition: The condition
    """
    patterns: List[str] = ()
    preferredPattern: str = ''
    condition: str = ''


@dataclass
class SaveViewModelEditAndSubmitInfo(TcBaseObj):
    """
    Structure represents the parameters required to save the edits on the input objects and submit them to a workflow
    process.
    
    :var obj: Object to save edit and submit to a workflow process.
    :var viewModelProperties: Information of view model properties to be saved.
    :var isPessimisticLock: If false, the operation takes optimistic lock on the object. In optimistic locking the
    object is locked (based on the object last saved date) before performing the actual save and lock is released after
    save is complete. If true, the object needs to be checked out before making any edits (pessimistic locking). The
    locking of object is done during loadViewModelForEditing operation.
    :var workflowData: The workflow information that the input object needs to submit to a workflow. Workflow
    information is stored in a name and value string map (string, list of strings).  If workflowData map is empty,
    objects will not be submitted to any workflow. Supported keys are:
    submitToWorkflow: Boolean Property to define that input object need to submit to workflow process or not. The
    values for the key are: "true" and "false". 
    processName: Process name string. 
    processDescription: Process description string. 
    processTemplate: Name of the process template to be used to create new workflow process. 
    processAssignmentList: Name of the process assignment list to use while creating new workflow process. 
    If property submitToWorkflow value is true and processTemplate value is empty, then workflow
    """
    obj: BusinessObject = None
    viewModelProperties: List[ViewModelProperty] = ()
    isPessimisticLock: bool = False
    workflowData: PropertyValues3 = None


@dataclass
class SaveViewModelEditAndSubmitOutput(TcBaseObj):
    """
    Structure containing information about each input object and whether it was saved and submitted to workflow
    successfully or not.
    
    :var clientId: Input string to uniquely identify the input, used primarily for error handling and output mapping.
    :var workflowProcess: Workflow template object for each input object that will be created after it is submitted to
    workflow successfully.
    """
    clientId: str = ''
    workflowProcess: BusinessObject = None


@dataclass
class SaveViewModelEditAndSubmitResponse(TcBaseObj):
    """
    Structure containing saveViewModelEditAndSubmit() operation response.
    
    :var output: A list of SaveEditAndSubmitOutput structures that will contain information about that object such as
    whether it is submitted to workflow or not. It will contain the workflow template for each input object.
    :var serviceData: Partial errors are returned in the Service Data.
    """
    output: List[SaveViewModelEditAndSubmitOutput] = ()
    serviceData: ServiceData = None


@dataclass
class TablePropStruct(TcBaseObj):
    """
    Structure containing ViewModel Objects and ViewModelPropertyDescriptors for table row objects
    
    :var tableRows: ViewModel objects list representing table rows
    :var propDescriptors: A list of ViewModelPropertyDescriptors for the table row ViewModel Objects.
    """
    tableRows: List[ViewModelObject] = ()
    propDescriptors: List[ViewModelPropertyDescriptor] = ()


@dataclass
class ViewData(TcBaseObj):
    """
    View data for GetViewerDataOutput.
    
    :var file: The ImanFile associated as a named reference to the dataset.
    :var fmsTicket: The read FMS ticket of ImanFile.
    :var viewer: Name of the viewer to render the file.
    """
    file: BusinessObject = None
    fmsTicket: str = ''
    viewer: str = ''


@dataclass
class ViewModelObject(TcBaseObj):
    """
    Object to store the Teamcenter BusinessObject and its associated ViewModel properties to be displayed in the view.
    
    :var modelObject: Represents the Model object.
    :var viewModelProperties: A list of ViewModelProperty structures.
    """
    modelObject: BusinessObject = None
    viewModelProperties: List[ViewModelProperty] = ()


@dataclass
class ViewModelProperty(TcBaseObj):
    """
    ViewModel property represents a property that is displayed in the view, it can be simple property, runtime
    property, compound property or dynamic compound property.
    
    :var propertyName: Property name.
    :var dbValues: Real value for the property.
    :var uiValues: Display values for the property.
    :var intermediateObjectUids: A list of UIDs of intermediate objects across the property traversal path.
    :var srcObjLsd: Last saved date of source object.
    :var isModifiable: The flag that indicates if the property is modifiable
    """
    propertyName: str = ''
    dbValues: List[str] = ()
    uiValues: List[str] = ()
    intermediateObjectUids: List[str] = ()
    srcObjLsd: datetime = None
    isModifiable: bool = False


@dataclass
class ViewModelPropertyDescriptor(TcBaseObj):
    """
    Represents the property descriptor details for a ViewModelProperty.
    
    :var srcObjectTypeName: Type name of the source object on which the property is defined.
    :var propertyName: Name of the property.
    :var valueType: Indicates the value type for the property. Valid value types are: 
    1 - Char
    2 - Date
    3 - Double
    4 - Float
    5 - Integer
    6 - Boolean
    7 - Short Integer
    8 - String
    9 - Type Reference
    10 - UnTyped Reference
    11 - External Reference
    12 - Note
    13 - Typed Relation
    14 - UnTyped Relation
    :var isArray: Has value as true in case of array properties and value as false for single valued properties.
    :var propConstants: A map(string,string) consisting of propertyConstant name as key and  and value being the
    property constant value.
    :var displayName: Display name of the property.
    :var lovCategory: Indicates the LOV category, if the property is attached to LOV.
    :var lov: LOV object reference associated with the property.
    :var maxArraySize: Maximimum number of elements allowed in case of VLA properties.
    :var maxLength: Max allowed length for the property
    :var propertyType: Indicates the property type. Valid property types are: 
    1 - attribute. 
    2 - Reference Property. 
    3 - Relation Property. 
    4 - Compound Property. 
    5 - Runtime Property.
    6 - Operation input.
    """
    srcObjectTypeName: str = ''
    propertyName: str = ''
    valueType: int = 0
    isArray: bool = False
    propConstants: PropertyConstantsMap = None
    displayName: str = ''
    lovCategory: int = 0
    lov: BusinessObject = None
    maxArraySize: int = 0
    maxLength: int = 0
    propertyType: int = 0


"""
Map of string to vector of strings. This map is used to return the auto assignable property name patterns to the client. The key is the property name and the value is the list of patterns.
"""
PatternMap3 = Dict[str, PatternInfo3]


"""
Map to store the property constant name and value pair.
"""
PropertyConstantsMap = Dict[str, str]


"""
Map (string, list of strings) that is a generic container that represents property values. The key is the property name and the value is the string representation of the property value.
"""
PropertyValues3 = Dict[str, List[str]]


"""
String map.
"""
StringMap7 = Dict[str, str]


"""
Map containing table property as key and corresponding table row information as value.
"""
TablePropObjectMap3 = Dict[str, TablePropStruct]


"""
Maps a business object to a thumbnail file ticket.
"""
ThumbnailMap6 = Dict[BusinessObject, str]



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class RevRuleEntryInfo(TcBaseObj):
    """
    Structure representing an entry\clause in a Revision Rule.
    
    :var entryType: Indicates type of the entry inside RevisionRule.A RevisionRule is made up of a sequential list of
    entries. Evaluation of the rule involves evaluating each of the entries, in order, until a configured revision of
    the Item is successfully obtained. A rule is made up of entries of the following types.
    0 - Working
    1 - Status
    2 - Override
    3 - Date
    4 - Unit No.
    5 - Group
    6 - Precise
    7 - Latest
    8 - End Item
    9 - GroupByItemType
    10 &ndash; Branch.
    :var displayText: Display text of the entry.
    :var revRuleEntryKeyToValue: A map (string, string) pairs of entry name and its value. For example:  1: For entry
    type Working, the map will contain "user", and "group" as keys and UIDs of User and Group as values respectively.
    2: For entry type Status, the map will contain "status_type"as key and UID of  attached status i.e  TaskType as
    value. 3: For entry type Override, the map will contain "folder" as key and UID of Folder as value.
    :var groupEntryInfo: Information about group entry.This structure is populated only in case of entry type is any of
    the supported group type entries.i.e. CFMGroupEntry, CFMGroupItemTypeEntry and Fnd0CFMGroupOccTypeEntry.
    """
    entryType: int = 0
    displayText: str = ''
    revRuleEntryKeyToValue: MapOfRevRuleEntryKeyToValue = None
    groupEntryInfo: RevRuleGroupEntryInfo = None


@dataclass
class RevRuleGroupEntryInfo(TcBaseObj):
    """
    Structure to hold an information about group entry.
    
    :var groupByTypes: A list containing display names of types of Item or PSOccurrence on which this rule entry is to
    be evaluated. This will be populated in cases of entry type is CFMGroupItemTypeEntry or Fnd0CFMGroupOccTypeEntry.
    Example: For CFMGroupItemTypeEntry, it can contain display names of item types as AllocationMap, CAEBCItem,
    CAEConnItem, CAEItem.
    :var listOfSubEntries: A list of entries that are grouped together.
    """
    groupByTypes: List[str] = ()
    listOfSubEntries: List[RevRuleSubEntryInfo] = ()


@dataclass
class RevRuleInfoResponse(TcBaseObj):
    """
    This structure contains all information about input RevisionRule like description, all clauses of RevisionRule,
    Boolean if RevisionRule supports nested effectivity.
    
    :var description: Description of a RevisionRule.
    :var entriesInfo: A list containing rule entries in a RevisionRule.
    :var nestedEffectivity: If true, nested effectivity is added to RevisionRule. If nested effectivity is added,
    Teamcenter evaluates nested effectivities as per effectivity mappings.
    """
    description: str = ''
    entriesInfo: List[RevRuleEntryInfo] = ()
    nestedEffectivity: bool = False


@dataclass
class RevRuleSubEntryInfo(TcBaseObj):
    """
    Structure represents a sub entry\clause in a group entry.
    
    :var subEntryType: Indiactes type of the subentry inside group entry.A group entry is made up of a sequential list
    of sub entries. Evaluation of the group entry involves evaluating each of the sub-entries in a given order.Valid
    types of sub-entries are:
    0 - Working 
    1 - Status 
    2 - Override 
    3 - Date 
    4 - Unit No. 
    6 - Precise 
    7 - Latest 
    8 - End Item 
    10 &ndash; Branch.
    :var displayText: Display text of the sub entry.
    :var mapOfSubEntryKeyToValue: A map (string, string) pairs of entry name and its value. For example, 
    1: For entry type Working, the map will contain "user", and "group" as keys and UIDs of User and Group as values
    respectively. 
    2: For entry type Status, the map will contain "status_type"as key and UID of  attached status i.e TaskType as
    value. 
    3: For entry type Override, the map will contain "folder" as key and UID of Folder as value.
    """
    subEntryType: int = 0
    displayText: str = ''
    mapOfSubEntryKeyToValue: MapOfRevRuleEntryKeyToValue = None


@dataclass
class RevisionRuleInfo(TcBaseObj):
    """
    Structure containing information about RevisionRule's name, description,clauses using which RevisionRule will be
    created or updated.
    
    :var uid: Unique Identifier (UID) of a RevisionRule to be created or updated.
    :var name: Name of the RevisionRule.
    :var description: Description of a RevisionRule.
    :var entriesInfo: A list containing entries in a RevisionRule.
    :var nestedEffectivity: If true, nested effectivity is added to RevisionRule. If nested effectivity is added,
    Teamcenter evaluates nested effectivities as per effectivity mappings.
    """
    uid: str = ''
    name: str = ''
    description: str = ''
    entriesInfo: List[RevRuleEntryInfo] = ()
    nestedEffectivity: bool = False


@dataclass
class CreateOrUpdateRevRuleResp(TcBaseObj):
    """
    Returns information on created or updated RevisionRule along with service data containing created or updated
    objects.
    
    :var revisionRuleInfo: Information about created or updated RevisionRule.
    :var serviceData: 'ServiceData' with created or updated objects.
    The following partial errors may be returned as part of service data:
    710001 An internal error has occurred in the Configuration Management module. Please report this error to your
    system administrator.
    710041 Revision Rule does not exist.
    710042 Revision Rule with name already exists.
    710052 Revision Rule already has a Date Entry - only one allowed per rule.
    710053 Revision Rule already has a Unit No Entry - only one allowed per rule. 
    710055 An Override Entry must contain a valid Override Folder.
    710056 Invalid date : must be set either to today or to a valid date. 
    710057 Invalid unit number : must be zero or positive.
    """
    revisionRuleInfo: RevisionRuleInfo = None
    serviceData: ServiceData = None


@dataclass
class CreateUpdateRevRuleInput(TcBaseObj):
    """
    Input structure required to create or update RevisionRule.
    
    :var updateExisting: If true, an existing RevisionRule with the same name will be updated else a new RevisionRule
    will be created.
    :var isPrivate: If true, RevisionRule is avaliable only to the user who has logged-in.In this case, a runtime
    property 'suppressed' on a RevisionRule will be set.A combination of properties 'suppressed' and 'owning_user' will
    be used to decide if a RevisionRule is avaiable to the user or not.
    :var revisionRuleInfo: Information on RevisionRule to be created or updated.
    """
    updateExisting: bool = False
    isPrivate: bool = False
    revisionRuleInfo: RevisionRuleInfo = None


"""
A map (string, string) pairs of entry name and its value. For example, 
1: For entry type Working, the map will contain "user", and "group" as keys and UIDs of User and Group as values respectively. 
2: For entry type Status, the map will contain "status_type"as key and UID of  attached status i.e TaskType as value. 
3: For entry type Override, the map will contain "folder" as key and UID of Folder as value.
"""
MapOfRevRuleEntryKeyToValue = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ConfiguratorServiceResponse(TcBaseObj):
    """
    Represents the response object for executeConfiguratorService operation. It contains the response XML from the
    configurator service.
    
    :var responseData: This contains the service response xml. Details of response schemas are listed at
    "//plm/tcpmm10.1.1/tcbom/configurator/xids"
    :var serviceData: This member is provided by the standard SOA framework for all operations. It contains information
    about error status and additional objects that were affected in this operation.
    """
    responseData: str = ''
    serviceData: ServiceData = None



from tcsoa.gen.Internal_DocMgmtAw._2019_06.services import DocMgmtService as imp0
from tcsoa.gen.Internal_DocMgmtAw._2019_12.services import DocMgmtService as imp1
from tcsoa.utils import TcService


class DocMgmtService(TcService):
    processMarkups = imp0.processMarkups
    processTextDataset = imp1.processTextDataset



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Group, ImanVolume
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AddChildGroupsToGroupStructure(TcBaseObj):
    """
    Groups to be added to a parent Group.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var groupsToAdd: A list of existing Group objects to be added to the parent Group as child groups.
    :var groupsToCreateAndAdd: A list of CreateAndAddChildGrpsToGrpStructure objects for Group. A new Group will be
    created for each input in the list and added to the parent Group.
    :var parentGroup: The parent Group the child groups are to be added to.
    """
    clientId: str = ''
    groupsToAdd: List[Group] = ()
    groupsToCreateAndAdd: List[CreateAndAddChildGrpsToGrpStructure] = ()
    parentGroup: Group = None


@dataclass
class CreateAndAddChildGrpsToGrpStructure(TcBaseObj):
    """
    Group to be created.
    
    :var name: Name of the Group to be created.
    :var description: Description of the Group.
    :var privilege: Indicates whether members of the Group will get administrative privileges.
    If true, all members of this Group will have system administration privilege; otherwise members will not have
    administration privilege.
    :var localVolume: Default local volume for the Group, local volume refers to a physical location with same site as
    Group where the Teamcenter files are stored. This temporary local volume allows the file to be stored locally
    before it is automatically transferred to the final destination in the background. Once the file is stored in the
    default local volume, the user can continue working without having to wait for the upload to take place.
    :var security: Specifies the project level security settings of Group. Valid values are "Internal" or "External".
    The internal/external Security setting allows or restricts access to data. For example, members of external groups
    can only access data in their group.
    :var volume: The location where the Teamcenter files to be stored by the users of this Group.
    """
    name: str = ''
    description: str = ''
    privilege: bool = False
    localVolume: ImanVolume = None
    security: str = ''
    volume: ImanVolume = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMLine
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetBOMLineInfo(TcBaseObj):
    """
    Structure represents the parameters required to get BOMLine after create.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var selectedBomLine: Parent BOMLine under which newly created BOMLine will be attached.
    :var newComp: Item for which BOMLine needs to be created.
    """
    clientId: str = ''
    selectedBomLine: BOMLine = None
    newComp: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import TaskDependency, ScheduleMember, POM_object, Schedule, Fnd0ProxyTask, ScheduleTask
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class LoadScheduleInfo(TcBaseObj):
    """
    Information required to load a Schedule based on a reference ScheduleTask and its parents.
    
    :var schedule: The Schedule to load.
    :var referenceTask: The reference ScheduleTask from where to start loading the subsequent ScheduleTask objects,
    which could be either siblings of reference ScheduleTask or siblings of one of the parents of reference
    ScheduleTask. If the reference ScheduleTask is null, the children of the parentTasks specified in the input will be
    loaded.
    :var parentTasks: List of all top level parents of the referenceTask, with immediate parent at the start of the
    list. If not specified, no ScheduleTask objects will be loaded.
    """
    schedule: Schedule = None
    referenceTask: ScheduleTask = None
    parentTasks: List[ScheduleTask] = ()


@dataclass
class LoadScheduleResponse(TcBaseObj):
    """
    Response of the load schedule operation, containing the list of loaded ScheduleTask, parent ScheduleTask,
    Fnd0ProxyTask and TaskDependency objects.
    
    :var scheduleTasks: List of ScheduleTask objects loaded.
    :var parentTasks: List of parents of ScheduleTask objects loaded. The order of parents will be same as that of the
    order of ScheduleTask objects loaded.
    :var proxyTasks: List of Fnd0ProxyTask objects referencing the loaded ScheduleTask objects.
    :var taskDependencies: List of TaskDependency objects relating the loaded ScheduleTask objects.
    :var hasMoreTasks: If true, there are more ScheduleTask objects in the Schedule, which are not loaded by this
    operation.
    :var dayRanges: Map (int, list of string) containing the ranges for each day in a week. Maps the days of the week
    to their ranges. The valid values for days are 0 to 6 which represents Sunday through Saturday in the respective
    order. A range is a pair of start and end time of working hours. A day can have multiple ranges with breaks
    in-between in a working day.
    :var eventRanges: The event dates and their respective ranges. The events are exceptions where the working hours
    are different from the default working hours.
    :var serviceData: The ServiceData.
    """
    scheduleTasks: List[ScheduleTask] = ()
    parentTasks: List[ScheduleTask] = ()
    proxyTasks: List[Fnd0ProxyTask] = ()
    taskDependencies: List[TaskDependency] = ()
    hasMoreTasks: bool = False
    dayRanges: DayRanges = None
    eventRanges: List[EventDateRanges] = ()
    serviceData: ServiceData = None


@dataclass
class ReplaceMemberAssignmentData(TcBaseObj):
    """
    The replaceMemberAssignmentData structure holds information needed to replace member assignments in the Schedule.
    
    :var oldResource: The ScheduleMember assignments to be replaced.
    :var newResource: The resource (User, Group, or Discipline) to be assigned replacing the oldResource.
    """
    oldResource: ScheduleMember = None
    newResource: POM_object = None


@dataclass
class EventDateRanges(TcBaseObj):
    """
    Maps an event date with its ranges. The number of ranges must be an even number as a range is represented as a pair
    of start and end time of working hours. The ranges will be empty if the event date is a non-working day.
    
    :var eventDate: The event date.
    :var ranges: An array of ranges for the event date. The number of ranges must be an even number as a range is
    represented as a pair of start and end time of working hours. A range must be represented in &lsquo;hh:mm&rsquo;
    format, where &lsquo;hh&rsquo; represents hours and &lsquo;mm&rsquo; represents minutes. For example, the array
    [08:30, 12:30, 13:45, 16:45] represents two ranges. The first range has start time of 08:30 hours and end time of
    12:30 hours. The second range has start time of 13:45 hours and end time of 16:45 hours.
    """
    eventDate: datetime = None
    ranges: List[str] = ()


"""
Maps the weekdays to their ranges. The valid values for the days are 0 to 6 which represents Sunday through Saturday in the respective order. A range is a pair of start and end time of working hours. A day can have multiple ranges with breaks in-between in a working day.
"""
DayRanges = Dict[int, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateTranslationRequestArgs(TcBaseObj):
    """
    The CreateTranslationRequestArgs struct is used to pass in multiple sets of data
    to be used in a single call.  These structs are passed in the collection of
    input arguments to the function createTranslationRequest.
    
    :var primaryObjects: The primary objects for the request.  This usually
    refers to a dataset to translate but can be any component.
    :var secondaryObjects: The secondary objects for the request.  This usually
    refers to the Item Revision containing the primary objects.
    :var priority: The priority to assign to the request.
    :var providerName: The provider name to process the request.
    :var translatorName: The translator from the above provider to translate the request.
    :var trigger: The trigger is a string that identifies who/where created this request.
    :var translatorArgs: The translator arguments to pass to the translator.  These are name
    value pairs like: NAME=FENDER.
    """
    primaryObjects: List[BusinessObject] = ()
    secondaryObjects: List[BusinessObject] = ()
    priority: int = 0
    providerName: str = ''
    translatorName: str = ''
    trigger: str = ''
    translatorArgs: List[str] = ()


@dataclass
class CreateTranslationRequestResponse(TcBaseObj):
    """
    The CreateTranslationRequestResponse struct contains the requests that were created
    as a result of the inputs specified in the CreateTranslationRequestArgs struct above.
    
    :var requestsCreated: The Translation Request objects created.
    :var svcData: The Service Data.
    """
    requestsCreated: List[BusinessObject] = ()
    svcData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Cad._2008_06.DataManagement import AttributeInfo
from tcsoa.gen.BusinessObjects import BusinessObject, Folder
from tcsoa.gen.Cad._2010_09.DataManagement import DatasetFileInfo
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExtraObjectInfo(TcBaseObj):
    """
    Form objects that can be created or updated and related to an item, item revision or dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var createOrUpdateInput: CreateOrUpdateInput object which contains attributes to create an extra object.
    createOrUpdateInput can be null for update.
    :var relationTypeName: Name of the relation type to the parent object
    """
    clientId: str = ''
    createOrUpdateInput: CreateOrUpdateInput = None
    relationTypeName: str = ''


@dataclass
class ItemInfo(TcBaseObj):
    """
    The 'ItemInfo' structure represents all of the data necessary to construct the item object. The basic attributes
    that are required are passed as named elements in the structure. All other attributes are passed as name/value
    pairs in the 'AttributeInfo' structures: 'attrList' and 'formAttrList'. The 'extraObject' field allows for the
    creation of an object(s) that will be related to this newly created Item.
    
    :var createOrUpdateInput: 'CreateOrUpdateInput' object which contains attributes to create an Item.
    'createOrUpdateInput' can be null for update.
    :var itemExtraObject: List of extra objects to be used or created and related to the Item.
    :var itemRevisionExtraObject: List of extra objects to be used or created and related to the Item Revision.
    :var folder: Folder to attach the Item to. If null, then the Teamcenter preference WsoInsertNoSelectionsPref is
    used to get the default location.
    """
    createOrUpdateInput: CreateOrUpdateInput = None
    itemExtraObject: List[ExtraObjectInfo] = ()
    itemRevisionExtraObject: List[ExtraObjectInfo] = ()
    folder: Folder = None


@dataclass
class NamedReferenceObjectInfo(TcBaseObj):
    """
    Contains information regarding named reference type value, object reference, object type name and list of attribute
    information to set on the object.
    
    :var clientId: Identifier defined by client to track the related object.
    :var createOrUpdateInput: Object that contains 'attributes' to create a named reference object.
    'createOrUpdateInput' can be null for update.
    :var namedReferenceName: The named reference from the dataset to this object and a required input. Named reference
    name values  are defined for each dataset type. The customer can add more values as needed. To get a current list
    of valid named reference name values the programmer can either use the Business Modeler IDE or can call the Core
    service 'getDatasetTypeInfo'.
    :var namedReferenceType: The reference type name from the dataset to this object, must be either AE_ASSOCIATION or
    AE_PART_OF.
    """
    clientId: str = ''
    createOrUpdateInput: CreateOrUpdateInput = None
    namedReferenceName: str = ''
    namedReferenceType: str = ''


@dataclass
class PartInfo(TcBaseObj):
    """
    The 'PartInfo' struct is the main input to the 'createOrUpdateParts' service for boundingbox. This structure refers
    to the item and one or more dataset structures used to create those objects.
    
    :var clientId: Identifier defined by the user to track the related object.
    :var itemInput: Member of type 'ItemInfo'.
    :var datasetInput: List of 'DatasetInfos'.
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    datasetInput: List[DatasetInfo] = ()


@dataclass
class CreateOrUpdateInput(TcBaseObj):
    """
    The 'CreateOrUpdateInput' structure represents all of the data necessary to construct a business object. All
    attributes are passed in as name/value pairs for the corresponding value type map.
    The 'compoundCreateOrUpdateInput' field allows for the creation of a secondary object(s) for the newly created
    primary object.
    
    :var boName: Business Object type name. This field must be specified for create.
    :var boReference: Existing object reference. This field must be specified for update.
    :var boolProps: A map (string/boolean) of property names to a single value.
    :var boolArrayProps: A map (string/list of booleans) of property names to a list of values.
    :var dateProps: A map (string/datetime) of property names to a single value.
    :var dateArrayProps: A map (string/list of datetimes) of property names to a list of values.
    :var tagProps: A map (string/business object) of property names to a single value.
    :var tagArrayProps: A map (string/list of business objects) of property names to a list of values.
    :var compoundCreateOrUpdateInput: 'CreateOrUpdateInput' for secondary (compounded) objects
    :var stringProps: A map (string/string) of property names to a single value.
    :var stringArrayProps: A map (string/list of strings) of property names to a list of values.
    :var doubleProps: A map (string/double) of property names to a single value.
    :var doubleArrayProps: A map (string/list of doubles) of property names to a list of values.
    :var floatProps: A map (string/float) of property names to a single value.
    :var floatArrayProps: A map (string/list of floats) of property names to a list of values.
    :var intProps: A map (string/integer) of property names to a single value.
    :var intArrayProps: A map (string/list of integers) of property names to a list of values.
    """
    boName: str = ''
    boReference: BusinessObject = None
    boolProps: BoolMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    tagProps: TagMap = None
    tagArrayProps: TagVectorMap = None
    compoundCreateOrUpdateInput: CreateOrUpdateInputMap = None
    stringProps: StringMap = None
    stringArrayProps: StringVectorMap = None
    doubleProps: DoubleMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None


@dataclass
class DatasetInfo(TcBaseObj):
    """
    The 'DatasetInfo' struct represents all of the data necessary to construct the Dataset object. The basic attributes
    that are required are passed as named elements in the structure. All other attributes are passed as name/value
    pairs in the 'AttributeInfo' structure: 'attrList'. The extraObject field allows for the creation of an object(s)
    that will be related to this newly created Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var createOrUpdateInput: 'CreateOrUpdateInput' object which contains attributes to create a Dataset.
    'createOrUpdateInput' can be null for update.
    :var datasetFileInfos: List of 'DatasetFileInfos', which holds the basic information for a file to be uploaded to a
    Dataset.
    :var namedReferenceObjectInfos: List of 'NamedReferenceObjectInfos', contains information regarding named reference
    type value, object reference, object type name and list of attribute information to set on the object.
    :var basisName: Basis name attribute value. When the dataset name is blank, the basis name is used to call
    USER_new_dataset_name to generate a new name.
    :var lastModifiedOfDataset: If not null, the date and time that the Dataset was last modified. If the actual
    modified date and time is later, then an error is thrown.
    :var itemRevRelationName: The relation name for the Dataset to ItemRevision relation. Can be null, but is required
    if an ItemRevision is specified.
    :var createNewVersion: Flag to create new version ( TRUE ) or not (FALSE ).
    :var mapAttributesWithoutDataset: Flag to indicate whether 'DatasetInfo' should be used for mapping attributes or
    for create.
    :var namedReferencePreference: Preference name which holds the list of named references to delete from one Dataset
    version to the next.
    :var mappingAttributes: List of 'AttributeInfos' for mapped attributes. Mapped attributes are attributes that are
    applied to other objects. Refer to the ITK manual for a definition of attribute mapping.
    :var extraObject: List of 'ExtraObjectInfos', the extra objects to be created and related to the Dataset.
    """
    clientId: str = ''
    createOrUpdateInput: CreateOrUpdateInput = None
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo] = ()
    basisName: str = ''
    lastModifiedOfDataset: datetime = None
    itemRevRelationName: str = ''
    createNewVersion: bool = False
    mapAttributesWithoutDataset: bool = False
    namedReferencePreference: str = ''
    mappingAttributes: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()


"""
Map of string client ids to vector of 'CreateOrUpdateInput' values (string, vector).
"""
CreateOrUpdateInputMap = Dict[str, List[CreateOrUpdateInput]]


"""
Map of DateTime property names to values (string, < DateTime>).
"""
DateMap = Dict[str, datetime]


"""
Map of DateTime array property names to values (string, vector< DateTime>).
"""
DateVectorMap = Dict[str, List[datetime]]


"""
DoubleMap
"""
DoubleMap = Dict[str, float]


"""
Map of double array property names to values (string, vector).
"""
DoubleVectorMap = Dict[str, List[float]]


"""
Map of float property names to values (string, float).
"""
FloatMap = Dict[str, float]


"""
Map of float array property names to values (string, vector).
"""
FloatVectorMap = Dict[str, List[float]]


"""
Map of int property names to values (string, int).
"""
IntMap = Dict[str, int]


"""
IntVectorMap
"""
IntVectorMap = Dict[str, List[int]]


"""
Map of boolean property names to values (string, bool).
"""
BoolMap = Dict[str, bool]


"""
Map of string property names to values (string, string).
"""
StringMap = Dict[str, str]


"""
Map of bool array property names to values (string, vector< bool >).
"""
BoolVectorMap = Dict[str, List[bool]]


"""
Map of string array property names to values (string, vector).
"""
StringVectorMap = Dict[str, List[str]]


"""
Map of BusinessObject property names to values (string, BusinessObject).
"""
TagMap = Dict[str, BusinessObject]


"""
Map of BusinessObject array property names to values (string, vector).
"""
TagVectorMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2008_06.DataManagement import CreateInput
from tcsoa.gen.Core._2013_05.DataManagement import DeepCopyData
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateIdInput(TcBaseObj):
    """
    - createInput    Holds property-value information for creating objects.
    - propertyName    Name of the Business Object's Property for which the ID is generated.
    - quantity    Number of IDs to be generated.
    
    
    
    :var createInput: Holds property-value information for Item and its subtypes.
    :var propertyName: Name of the Business Object's Property for which the ID is generated.
    :var quantity: Number of IDs to be generated.
    """
    createInput: CreateInput = None
    propertyName: str = ''
    quantity: int = 0


@dataclass
class GenerateIdResponseStruct(TcBaseObj):
    """
    Structure for holding GenerateIdResponse for each CreateIdInput.
    
    :var generatedIDs: List of generated IDs.
    """
    generatedIDs: List[str] = ()


@dataclass
class GenerateIdsResponse(TcBaseObj):
    """
    - generateIdsOutput : Multiple lists of Generated IDs. Each list corresponds to the individual elements in
    GenerateIdInputs
    - serviceData : Holds error stacks
    
    
    
    :var generateIdsOutput: Multiple lists of Generated IDs. Each list corresponds to the individual elements in
    GenerateIdInputs.
    :var serviceData: Holds error stacks.
    """
    generateIdsOutput: List[GenerateIdResponseStruct] = ()
    serviceData: ServiceData = None


@dataclass
class GetDeepCopyDataResponse(TcBaseObj):
    """
    Structure that contains the DeepCopyData embedded within which is a recursive data structure. The DeepCopyData
    contains information about how the secondary objects (related and referenced objects) are to be copied (reference,
    copy as object, no copy).
    
    :var deepCopyInfoMap: A map of business objects and DeepCopyData (POM_object/DeepCopyData). Each business object
    from the method input will have a DeepCopyData in the map. The DeepCopyData object contains all the information
    about how the attached objects are to be copied (Copy as Object, Copy as Reference, NoCopy, etc.). DeepCopyData is
    a recursive data structure that contains the details for the attached objects at the next level.
    :var serviceData: Service data containing errors, etc. The plain object list of the Service data is populated with
    the target objects which are to be copied as part of the SaveAs/revise operation. If there is an error retrieving
    Business Object for the business object name corresponding to the target object, it is added to the error stack of
    the ServiceData as a partial error
    """
    deepCopyInfoMap: DeepCopyInfoMap = None
    serviceData: ServiceData = None


@dataclass
class GetPasteRelationsOutput2(TcBaseObj):
    """
    The struct data to be returned. This struct contains a clientId, a list of PasteRelationInfo2 instances, and a
    preferred index of relation in the list pasreRelInfo.
    
    :var clientId: Unique client identifier
    :var pasteRelInfo: A list of paste relations info to be returned
    :var indexOfPreferred: The zero-based index of the preferred paste relation in the list pasreRelInfo.
    
    """
    clientId: str = ''
    pasteRelInfo: List[PasteRelationsInfo2] = ()
    indexOfPreferred: int = 0


@dataclass
class GetPasteRelationsResponse2(TcBaseObj):
    """
    Response structure contains a list of GetPasteRelationsOutput2 and ServiceData
    
    :var outputs: A list of paste relation output.
    :var serviceData: 'Service Data' including partial errors that are mapped to the client id from the input. Created
    objects are also added to the created objects list in the 'Service Data'.
    """
    outputs: List[GetPasteRelationsOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class PasteRelationsInfo2(TcBaseObj):
    """
    This struct contains the internal name and display name of the relation as well as a flag indicating whether the
    relation will enable the children to be shown under the parent when expanded.
    
    :var pastRelName: The internal name of paste relation
    :var pastRelDisplayName: The displayname of paste relation
    :var isExpandable: If true, will enable the children to be shown under the parent when expanded.
    
    """
    pastRelName: str = ''
    pastRelDisplayName: str = ''
    isExpandable: bool = False


@dataclass
class SaveAsIn(TcBaseObj):
    """
    This structure holds information about the target object, a map of the property name as key and as a value carries
    a list of the corresponding values for that property name and the deep copy data of the objects attached to the
    target object.
    
    :var targetObject: The target business object.
    :var inputPropValues: A map of property name (as key) and property values (as value) in string format. Each value
    is a list of strings to support both single valued and multi valued properties of types. The calling client is
    responsible for converting the different property types (like integer, double, date, etc) to a string using the
    appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
    :var deepCopyDatas: DeepCopyData of the objects attached to the target object.
    """
    targetObject: BusinessObject = None
    inputPropValues: PropertyValuesMap = None
    deepCopyDatas: List[DeepCopyData] = ()


@dataclass
class ChildrenInputData(TcBaseObj):
    """
    This is the input structure which will be used by the AddChildren and RemoveChildren service operations.
    
    :var clientId: An unique string supplied by the caller. This Id is used by the response to identify the failed
    add/remove children.
    
    :var parentObj: A parent object to which the child objects would be added or removed.
    :var childrenObj: List of objects which are added or removed as children to parent object.
    :var propertyName: The name of  the property that relates  the child objects to the parent. 
    The property can be a relation property or reference property or empty.
    """
    clientId: str = ''
    parentObj: BusinessObject = None
    childrenObj: List[BusinessObject] = ()
    propertyName: str = ''


@dataclass
class DeepCopyData(TcBaseObj):
    """
    DeepCopyData stores the deep copy information that will be copied via saveAs or revise operation. It also stores
    the nested deep copy data at the next level. 
    
    :var attachedObject: The related object to be deep copied.
    :var propertyValuesMap: Map of DeepCopyData ( string, list of strings ). It can contain the following attributes:
    
    - isRequired If true, the copy action can not be modified. If false, the copy action can be changed different
    action by the user.
    - propertyName Name of relation type or reference property for which DeepCopy rule is configured.
    - copyAction DeepCopy action Supported values are as following: NoCopy, CopyAsReference, CopyAsObject or Select.
    - isTargetPrimary If true the target object is processed as primary, otherwise it is processed as a secondary
    object.
    - copy_relations If true, the custom properties on the source relation object are copied over to the newly-created
    relation. If false, custom properties are not copied.
    - secondaryIsDuplicated if true, the attached object already appears in deep copy data
    - propertyType If Relation, it represents Relation type property. If Reference, it represents Reference property.
    
    
    :var operationInputTypeName: Object type name of the operation being perfomed
    :var childDeepCopyData: List of DeepCopyData for the objects of the relation or reference property objects of the
    attached object.
    :var operationInputs: Map to provide input property names and values of attached object. These property values will
    be applied on copied object. Map of property name(key) and property values(values) (string, list of strings) in
    string format of attached object, to be set on copied object of attached object. The calling client is responsible
    for converting the different property types (int, float, date .etc) to a string using the appropriate toXXXString
    functions in the SOA client framework Property class.
    """
    attachedObject: BusinessObject = None
    propertyValuesMap: PropertyValuesMap = None
    operationInputTypeName: str = ''
    childDeepCopyData: List[DeepCopyData] = ()
    operationInputs: PropertyValuesMap = None


@dataclass
class DeepCopyDataInput(TcBaseObj):
    """
    Input structure for getDeepCopyData operation
    
    :var operation: This is the operation types such as SaveAs,Revise, etc.
    :var businessObject: object reference to get the deepcopydata
    """
    operation: str = ''
    businessObject: BusinessObject = None


"""
Map contains a list of pairs (BusinessObject whose type is POM_object or its subtype, list). For each pair, name is the business object and value holds deep copy data for the business object.
"""
DeepCopyInfoMap = Dict[BusinessObject, List[DeepCopyData]]


"""
This map is of property name (as key) and property values (as value) in string format. Each value is a list of strings to support both single valued and multi valued properties of types. The calling client is responsible for converting the different property types (like integer, double, date, etc) to a string using the appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
"""
PropertyValuesMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2010_09.StructureVerification import AccountabilityCheckResult
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AccountabilityCheckResponse(TcBaseObj):
    """
    Contains all the results from the accountabilityCheck operation
    
    :var accountabilityCheckResults: A vector of accountability check results
    :var reachableTargets: A vector of reachable target lines
    :var serviceData: The service data
    :var sourceConfigContext: The source configuration context for an equivalent source object.
    :var targetConfigContext: The target configuration context for an equivalent target object.
    """
    accountabilityCheckResults: List[AccountabilityCheckResult] = ()
    reachableTargets: List[BusinessObject] = ()
    serviceData: ServiceData = None
    sourceConfigContext: BusinessObject = None
    targetConfigContext: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class GenerateAndEvaluateStructureResponse(TcBaseObj):
    """
    Response structure for generateAndEvaluateStructure method.
    
    :var xpathToValuesMap: The map of xpath to the values obtained by evaluating this xpath against the generated
    plmxml.
    :var serviceData: partial failures are returned - along with object ids for each plmxml data could not be generated.
    :var transientFileTicket: If an xml is generated on server - input does not specify an existing plmxml or invalid
    filename - a fileticket is returned for that file.
    """
    xpathToValuesMap: XpathToValuesMap = None
    serviceData: ServiceData = None
    transientFileTicket: str = ''


"""
Map of xpath string to the values obtained by running this against the plmxml file.
"""
XpathToValuesMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class InputObjectsStructure(TcBaseObj):
    """
    Structure containing selected input ItemRevision objects on which pre-configured simulation tool needs to be
    launched.
    
    :var inputItemRevisions: Array of selected ItemRevision objects on which pre-configured simulation tool needs to be
    launched.
    """
    inputItemRevisions: List[ItemRevision] = ()



from __future__ import annotations

from tcsoa.gen.ProjectManagementAw._2015_07.ScheduleManagementAw import CreateTaskDeliverablesResponse
from tcsoa.gen.BusinessObjects import Discipline, User, Schedule
from typing import List
from tcsoa.gen.ProjectManagementAw._2018_12.ScheduleManagementAw import TaskDeliverableInput, LoadScheduleInfo2, LoadScheduleResponse2, LoadBaselineInfo, RecalculateInput, LoadBaselineResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementAwService(TcService):

    @classmethod
    def loadSchedule2(cls, loadScheduleInfo: LoadScheduleInfo2) -> LoadScheduleResponse2:
        """
        Loads the ScheduleTask objects of a given Schedule based on the reference ScheduleTask. The number of
        ScheduleTask objects to be loaded will be determined by AWC_DefaultPageSize preference or &lsquo;50&rsquo;, if
        the preference is not found. This operation also loads the baseline tasks, Fnd0ProxyTask objects and
        TaskDependency relations which are referencing the loaded ScheduleTask objects.
        """
        return cls.execute_soa_method(
            method_name='loadSchedule2',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'loadScheduleInfo': loadScheduleInfo},
            response_cls=LoadScheduleResponse2,
        )

    @classmethod
    def recalculateSchedules(cls, recalculateInputs: List[RecalculateInput], runInBackground: bool) -> ServiceData:
        """
        This operation performs the revalidation and rerunning of the business logic on the properties of the schedule
        and its child objects based on the requested properties flag or ALL.
        """
        return cls.execute_soa_method(
            method_name='recalculateSchedules',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'recalculateInputs': recalculateInputs, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def recalculateSchedulesAsync(cls, recalculateInputs: List[RecalculateInput]) -> None:
        """
        This operation performs the revalidation and rerunning of the business logic on the properties of the schedule
        and its child objects based on the requested properties flag or ALL. This operation runs asynchronously in its
        own server in the background.
        """
        return cls.execute_soa_method(
            method_name='recalculateSchedulesAsync',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'recalculateInputs': recalculateInputs},
            response_cls=None,
        )

    @classmethod
    def createMultipleTaskDeliverables(cls, taskDeliverableInputs: List[TaskDeliverableInput]) -> CreateTaskDeliverablesResponse:
        """
        This operation takes in an instance of Item, Form, Folder, Dataset and Document and creates one or more
        ScheduleTaskDeliverables and ScheduleDeliverable and relates them to one or more ScheduleTask and its
        associated Schedule.
        """
        return cls.execute_soa_method(
            method_name='createMultipleTaskDeliverables',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'taskDeliverableInputs': taskDeliverableInputs},
            response_cls=CreateTaskDeliverablesResponse,
        )

    @classmethod
    def designateDiscipline(cls, schedule: Schedule, discipline: Discipline, user: User, revert: bool, runInBackground: bool) -> ServiceData:
        """
        This operation removes all assignments in a schedule of a Discipline and assigns them to the specified User.
        While designating the discipline, the system will check if the user is a ScheduleMember. If not, the system
        will create a ScheduleMember for the user. 
        The system will get and delete all resources assignments for the discipline in the schedule and create new
        resource assignments for the user.
        The revert option is to revert the ResourceAssignment objects from the User to Discipline object for all
        schedule tasks in the schedule.
        """
        return cls.execute_soa_method(
            method_name='designateDiscipline',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'schedule': schedule, 'discipline': discipline, 'user': user, 'revert': revert, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def designateDisciplineAsync(cls, schedule: Schedule, discipline: Discipline, user: User, revert: bool) -> None:
        """
        This operation removes all assignments in a schedule of a Discipline and assigns them to the specified User.
        While designating the discipline, the system will check if the user is a ScheduleMember. If not, the system
        will create a ScheduleMember for the user. 
        The system will get and delete all resources assignments for the discipline in the Schedule and create new
        resource assignments for the user.
        The revert option is to revert the ResourceAssignment objects from the User to Discipline object for all
        schedule tasks in the Schedule.
        """
        return cls.execute_soa_method(
            method_name='designateDisciplineAsync',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'schedule': schedule, 'discipline': discipline, 'user': user, 'revert': revert},
            response_cls=None,
        )

    @classmethod
    def loadBaseline(cls, loadBaselineInfo: LoadBaselineInfo) -> LoadBaselineResponse:
        """
        Loads the information about the baseline tasks of the given ScheduleTask objects based on the source Schedule
        and the baseline Schedule.
        """
        return cls.execute_soa_method(
            method_name='loadBaseline',
            library='ProjectManagementAw',
            service_date='2018_12',
            service_name='ScheduleManagementAw',
            params={'loadBaselineInfo': loadBaselineInfo},
            response_cls=LoadBaselineResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class TemplateFileInput(TcBaseObj):
    """
    Holds the template information for the 'getTemplateFiles' operation
    
    :var templateDirectory: A directory containing the template related file.
    :var templateFile: A template related file.
    """
    templateDirectory: str = ''
    templateFile: str = ''


@dataclass
class TemplateFileOutput(TcBaseObj):
    """
    Holds output data for a given template related file.
    
    :var templateDirectory: A directory containing template related files.
    :var templateFile: A template related file.
    :var templateFileTicket: This ticket is for the file 'templateFile' located at 'templateDirectory' on server.
    :var timestamp: The datetime stamp of the template related file.
    """
    templateDirectory: str = ''
    templateFile: str = ''
    templateFileTicket: str = ''
    timestamp: datetime = None


@dataclass
class TemplateFilesResponse(TcBaseObj):
    """
    Holds the response data of the 'getTemplateFiles' operation.
    
    :var outputs: A list of 'TemplateFileOutput' structures that contain the set of template data retrieved for each
    specified input 'TemplateFileInput'.
    :var serviceData: Status of the operation. Partial error may include following error codes
    - 216012 : Input directory not found
    
    """
    outputs: List[TemplateFileOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ChangedTemplateFileInput(TcBaseObj):
    """
    Input data for a given template related file.
    
    :var templateDirectory: A directory in Teamcenter Server installation containing the template related file. All
    template files are stored in $TC_DATA\model directory.
    :var templateFile: A template related file.
    :var timestamp: The datetime stamp of the templateFile to use for comparison with the datetime stamp of that same
    file in the OS.
    """
    templateDirectory: str = ''
    templateFile: str = ''
    timestamp: datetime = None


@dataclass
class DataModelDeploymentInput(TcBaseObj):
    """
    'DataModelDeploymentInput' structure represents parameters required for deploying data model.
    
    :var deploymentFileTicket: FMS ticket for a data model XML file of a type specified by 'deploymentFileType'.
    
    :var deploymentFileType: The type of file. The type can be '0, 1 or 2'. The value '0' is for template file, '1' is
    for template dependency file and '2' is for template language file.
    """
    deploymentFileTicket: str = ''
    deploymentFileType: int = 0



from __future__ import annotations

from tcsoa.gen.Manufacturing._2012_02.DataManagement import GeneralInfo, FileTicket, AssociatedContextInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ProcResourceResponseInfo(TcBaseObj):
    """
    The response contains a structure of the data related to process resource. The following partial errors may be
    returned:
    
    - 253127 - The information for this object cannot be fetched as the object is not in the  context of Plant Bill Of
    Processes (BOP). 
    
    
    :var infoMap: Map of BOMLine objects (business object/list of business objects).
    If the key is process area of type Mfg0BvrProcessArea then value is list of process resources of type
    Mfg0BvrProcessResource that are defined under the process area and process/operation of type
    Mfg0BvrProcess/Mfg0BvrOperation that are unallocated to process resource. The value may also contain children of
    the process area such as process line or station.
    
    If the key is process resource of type Mfg0BvrProcessResource then values are process/operation of type
    Mfg0BvrProcess/Mfg0BvrOperation that are allocated to process resource.
    
    If the key is process of type Mfg0BvrProcess then values are process/operation of type 
    Mfg0BvrProcess/Mfg0BvrOperation that are child of that process. These childs are process/operation that are
    unallocated to any process resource. If the process itself is allocated to process resource then the value is empty
    list.
    
    :var serviceData: The service data containing partial errors.
    """
    infoMap: ProcResourceRelatedInfoMap = None
    serviceData: ServiceData = None


@dataclass
class CloneAssemblyInputData(TcBaseObj):
    """
    Input to the service that is a source object that should be cloned, target parent object for the cloned object and
    additional parameters needed for functionality.
    
    
    :var sourceObject: Business object of the source structure to be cloned. The expected buisness object must be of
    type Item, ItemRevision or BOMLine.
    :var scopeSearchObject: Business object of the scope object to search in for parts to be replaced. The expected
    buisness object must be of type Item, ItemRevision or BOMLine. If not specified, the source object will be used.
    :var parentObject: Business object of the parent object for the new created clone object. The expected buisness
    object must be of BOMLine type. If not specified, the parent of the sourceObject will be used.
    :var sourceOccEff: Occurrence effectivity of the source object.
    :var newCloneOccEff: Occurrence effectivity of the new cloned object.
    :var additionalInfo: Additional information needed to create the new cloned object like name, revision, description
    etc.
    """
    sourceObject: BusinessObject = None
    scopeSearchObject: BusinessObject = None
    parentObject: BusinessObject = None
    sourceOccEff: str = ''
    newCloneOccEff: str = ''
    additionalInfo: GeneralInfo = None


@dataclass
class CloneAssemblyResponse(TcBaseObj):
    """
    output structure of cloneAssemblyAndProcessObjects SOA
    
    :var clonedObject: Business object of new BOMLine object, created as a result of clone operation under the new
    target.
    :var serviceData: Standard Service Data
    """
    clonedObject: BusinessObject = None
    serviceData: ServiceData = None


@dataclass
class AddOrRemoveContextsInfo(TcBaseObj):
    """
    The input parameter to the operation is the target context to associate to, list of source contexts and the
    relation name to associate or disassociate with and the additional action required for association or
    disassociation.
    
    :var context: The target context to associate or disassociate with e.g. Mfg0BvrProductBOP or Mfg0BvrPlantBOP.
    :var addContext: The list of source contexts , e.g  Mfg0BvrGenericBOP or Mfg0BvrProductBOP, that should be
    associated with the target context with the specified relation.
    :var removeContext: The list of source contexts, e.g. Mfg0BvrGenericBOP or Mfg0BvrProductBOP, for which the
    association with the target context should be removed with the specified relation.
    :var actionMap: A map (string, list of string) of context and related actions. The data allows additional action
    required for the add or remove context. e.g. the key of the map will be "RemoveContext" and "AddContext" and the
    values for the key such as for the key "RemoveContext" the action will be "RemoveLink" and "RemoveAllocation".
    "RemoveLink" will remove the link between the allocations.
    "RemoveAllocation" will remove the allocations.
    """
    context: BusinessObject = None
    addContext: List[AssociatedContextInfo] = ()
    removeContext: List[AssociatedContextInfo] = ()
    actionMap: ActionMap = None


@dataclass
class EstablishOriginLinkCandidates(TcBaseObj):
    """
    Source and targets that are candidates to link
    
    :var sourceObject: The source object is either operation of type Mfg0BvrOperation or process of type Mfg0BvrProcess.
    :var targetObjects: The list of target object and its linked state.
    """
    sourceObject: BusinessObject = None
    targetObjects: List[EstablishOriginLinkTargetLinkState] = ()


@dataclass
class EstablishOriginLinkCandidatesInput(TcBaseObj):
    """
    Source and target candidates to establish origin link.
    
    :var sourceObject: Operation of type Mfg0BvrOperation or process of type Mfg0BvrProcess from Generic BOP or Product
    BOP structure. Same object type from target objects are considered to establish the origin link.
    :var targetObjects: Operation of type Mfg0BvrOperation or process of type Mfg0BvrProcess to be linked. The objects
    are in the context of Product BOP or Plant BOP.
    """
    sourceObject: BusinessObject = None
    targetObjects: List[BusinessObject] = ()


@dataclass
class EstablishOriginLinkInfo(TcBaseObj):
    """
    Input to the service is a list of a source and targets to be linked.
    
    :var inputObjects: soure and target candidates to establish origin link.
    :var criteria: The list of criteria based on which the origin link established. Currently, the only criteria
    supported is logical designator property. Possible value of parameter is "LD" signifying that all source and target
    objects with matching logical designator property are considered for the operation.
    :var considerHierarchy: Specifies whether hierarchy of source and target should be considered while establishing
    the origin link. If the value is true then child of source and target are linked based on the criteria. If false
    then child of source and target is not considered.
    :var action: Specifies whether origin link should be established between source and target. Possible values are
    "DryRun" and "Link". If value is "DryRun" then candidates are fetched without establishing the origin link. If
    value is "Link" then the source and target are linked based on criteria specified.
    """
    inputObjects: List[EstablishOriginLinkCandidatesInput] = ()
    criteria: List[str] = ()
    considerHierarchy: bool = False
    action: str = ''


@dataclass
class EstablishOriginLinkResponse(TcBaseObj):
    """
    The response contains service data and list of source and target candidates that are linked.
    
    :var candidates: The source and target objects with their linked state.
    :var serviceData: The ServiceData contains partial errors if any.
    :var logFileTicket: The file ticket containing the UID and file name for the log file generated for this command.
    """
    candidates: List[EstablishOriginLinkCandidates] = ()
    serviceData: ServiceData = None
    logFileTicket: FileTicket = None


@dataclass
class EstablishOriginLinkTargetLinkState(TcBaseObj):
    """
    This structure contains the target object and its linked state with the provided source.
    
    :var targetObject: Target object is either operation of type Mfg0BvrOperation or process of type Mfg0BvrProcess.
    :var linkState:  Flag specifying whether target is linked with the provided source.
    """
    targetObject: BusinessObject = None
    linkState: bool = False


"""
The map (string, list of string) of additional actions required for add or remove contexts operation.
"""
ActionMap = Dict[str, List[str]]


"""
Map of BOMLine objects (business object/list of business objects).
If the key is process area of type Mfg0BvrProcessArea then value is list of process resources of type Mfg0BvrProcessResource that are defined under the process area and process/operation of type Mfg0BvrProcess/Mfg0BvrOperation that are unallocated to process resource. The value may also contain children of the process area such as process line or station.

If the key is process resource of type Mfg0BvrProcessResource then values are process/operation of type Mfg0BvrProcess/Mfg0BvrOperation that are allocated to process resource.

If the key is process of type Mfg0BvrProcess then values are process/operation of type 
Mfg0BvrProcess/Mfg0BvrOperation that are child of that process. These childs are process/operation that are unallocated to any process resource. If the process itself is allocated to process resource then the value is empty list.

"""
ProcResourceRelatedInfoMap = Dict[BusinessObject, List[BusinessObject]]



from tcsoa.gen.Allocations._2007_01.services import AllocationService as imp0
from tcsoa.gen.Allocations._2011_06.services import AllocationService as imp1
from tcsoa.utils import TcService


class AllocationService(TcService):
    addAllocationLines = imp0.addAllocationLines
    changeAllocatedBOMWindows = imp0.changeAllocatedBOMWindows
    changeAllocationContext = imp0.changeAllocationContext
    changeICContext = imp0.changeICContext
    closeAllocationWindow = imp0.closeAllocationWindow
    createAllocationContext = imp0.createAllocationContext
    createAllocationContext2 = imp1.createAllocationContext2
    deleteAllocationLines = imp0.deleteAllocationLines
    findAllocatedBOMViews = imp0.findAllocatedBOMViews
    findAllocationContexts = imp0.findAllocationContexts
    getBOMViews = imp0.getBOMViews
    modifyAllocationLines = imp0.modifyAllocationLines
    openAllocationWindow = imp0.openAllocationWindow
    saveAllocationWindow = imp0.saveAllocationWindow



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Role, User, TC_Project, GroupMember, Group
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.Core._2018_11.ProjectLevelSecurity import UserGroupRoleInfo
from dataclasses import dataclass


@dataclass
class GroupMemberPrivilege(TcBaseObj):
    """
    The structure holds the GroupMember object and its privilege in the ProjectTeam of the input TC_Project.
    
    :var groupmember: The GroupMember object
    :var privilege: The privilege status of the GroupMember in the ProjectTeam of the input TC_Project.
    0 = regular member
    1 = privileged member
    2 = project team administrator
    3 = project administrator
    :var isRemovable: Indicates if the GroupMember is removable from the ProjectTeam. If true, it indicates the
    GroupMember is directly in the ProjectTeam, and it is removable from the ProjectTeam. If false, it indicates the
    GroupMember is under a Group in the ProjectTeam, and it is not removable from the ProjectTeam.
    """
    groupmember: GroupMember = None
    privilege: int = 0
    isRemovable: bool = False


@dataclass
class GroupNode(TcBaseObj):
    """
    The view model for Group, which contains the Group object itself and a flag to indicate if the Group is removable
    from the ProjectTeam.
    
    :var tcGroup: The Group node view model
    :var isRemovable: If true, the Group is directly in the ProjectTeam and is removable. If false, the Group is a
    child of another Group and is not removable.
    """
    tcGroup: Group = None
    isRemovable: bool = False


@dataclass
class GroupNodeWithChildren(TcBaseObj):
    """
    The structure of a Group node with its child nodes.
    
    :var groupNode: The GroupNode.
    :var childGroups: The child Groups.
    :var childRoles: The Roles under the Group.
    """
    groupNode: GroupNode = None
    childGroups: List[GroupNode] = ()
    childRoles: List[GroupRoleNodeWithChildren] = ()


@dataclass
class GroupRoleNode(TcBaseObj):
    """
    The view model of a Role object, which contains the Role object it self, the Group object it is associated with,
    and a flag indicating if it is removable from the Project Team.
    
    :var tcGroup: The Group.
    :var tcRole: The Role.
    :var isRemovable: For the first level GroupRoleNode structures, they are removable because these are the parent
    node for the GroupMembers that are directly in the project members. Hence the values are true. For the
    GroupRoleNode structures under the Group objects in the ProjectTeam, the values are false.
    """
    tcGroup: Group = None
    tcRole: Role = None
    isRemovable: bool = False


@dataclass
class GroupRoleNodeWithChildren(TcBaseObj):
    """
    The structure which holds the first level node (Group.Role) and the GroupMember objects along with their privilege
    in the Project Team.
    
    :var groupRole: The first level Group.Role node.
    :var groupmemberList: The GroupMember objects that are in the Project Team with the same Group and Role, along with
    their privilege.
    """
    groupRole: GroupRoleNode = None
    groupmemberList: List[GroupMemberPrivilege] = ()


@dataclass
class ProjectAndPrivilege(TcBaseObj):
    """
    The structure to hold the TC_Project object and the privilege of the user.
    
    :var project: The TC_Project object retrieved from DB.
    :var privilege: The User's privilege in the associated TC_Project.
    - -1 = User is not a member of the project (possible when having bypass for DBA users).
    -  0 = User is a regular member of the project.
    -  1 = User is a project author of the project.
    -  2 = User is a project team administrator of the project.
    -  3 = User is the owning user of the project.
    
    """
    project: TC_Project = None
    privilege: int = 0


@dataclass
class ProjectPrivilegeResponse(TcBaseObj):
    """
    This structure contains a list of ProjectAndPrivilege structure that has privilege of the login user in each
    TC_Project object, and the Service Data.
    
    :var projectPrivilege: A list of TC_Project objects and the privileges of the login user in them.
    :var serviceData: The Service Data
    """
    projectPrivilege: List[ProjectAndPrivilege] = ()
    serviceData: ServiceData = None


@dataclass
class ProjectTeamPagedInput(TcBaseObj):
    """
    The paginated input to get the ProjectTeam for the given TC_Project object.
    
    :var project: The TC_Project object to get the ProjectTeam.
    :var startIndex: The start index
    :var pageSize: The page size
    :var quickLoad: If true, to retrieve the ProjectTeam data from the server memory. If false, the data is retrieved
    from the database.
    """
    project: TC_Project = None
    startIndex: int = 0
    pageSize: int = 0
    quickLoad: bool = False


@dataclass
class ProjectTeamPagedResponse(TcBaseObj):
    """
    The structure holds the paginated first level nodes of ProjectTeam, starting with Group objects.
    
    :var totalMemberCount: The total number of the project members in the ProjectTeam of the given TC_Project object.
    :var endIndex: The index of the last element returned for pagination.
    :var groups: A list of Group objects that are in the project members of the input TC_Project.
    :var structuredGroupMembers: A list of structured GroupRoleNode which represents the first level node of the
    GroupMember objects in the ProjectTeam.
    :var serviceData: The Service Data
    """
    totalMemberCount: int = 0
    endIndex: int = 0
    groups: List[GroupNode] = ()
    structuredGroupMembers: List[GroupRoleNode] = ()
    serviceData: ServiceData = None


@dataclass
class SetPrivilegeForUserInput(TcBaseObj):
    """
    The input structure for setUserPrivilege operation
    
    :var project: The TC_Project object which the User privilege needs to be set.
    :var users: The User objects to set privilege.
    :var groupNode: The Group node to set privilege on all the User objects under it. The privilege will be set for all
    the User objects under this Group, including subgroups.
    :var groupRoleNode: The GroupRole node to set privilege on all the User objects under it. The privilege will be set
    on all the User objects under this GroupRole node
    :var privilegeStatus: - 0 = regular member (non-privileged member)
    - 1 = project author (privileged member)
    - 2 = project team administrator
    
    """
    project: TC_Project = None
    users: List[User] = ()
    groupNode: List[GroupNode] = ()
    groupRoleNode: List[GroupRoleNode] = ()
    privilegeStatus: int = 0


@dataclass
class UserProjectsAndPrivilege(TcBaseObj):
    """
    The structure holds the User and the TC_Project objects that are associated with this User, along with the User's
    privilege status in each TC_Project.
    
    :var userGroupRole: The given User or the given User with specific Group and Role that the TC_Project objects are
    associated with.
    :var totalProjectCount: The total number of TC_Project associtated with the User.
    :var endIndex: The end index of the paged response.
    :var projects: All the TC_Project objects that are associated with the given User, along with the privilege status
    of the User.
    """
    userGroupRole: UserGroupRoleInfo = None
    totalProjectCount: int = 0
    endIndex: int = 0
    projects: List[ProjectAndPrivilege] = ()


@dataclass
class UserProjectsAndPrivilegeResponse(TcBaseObj):
    """
    The structure holds the User objects, the associated TC_Project objects, and their privilege status in each
    TC_Project objects.
    
    :var userProjects: The Users and their associated TC_Projects, along with the privilege status of each User.
    :var serviceData: The Service Data.
    """
    userProjects: List[UserProjectsAndPrivilege] = ()
    serviceData: ServiceData = None


@dataclass
class ChildStructureResponse(TcBaseObj):
    """
    The structure to hold the children of a specific node in ProjectTeam.
    
    :var childGroups: The child Groups.
    :var childRoles: The child Roles.
    :var childGroupMembers: The child GroupMembers.
    :var childGroupMap: Map the Group objects to their child Groups.
    :var sd: The Service Data.
    """
    childGroups: List[GroupNodeWithChildren] = ()
    childRoles: List[GroupRoleNodeWithChildren] = ()
    childGroupMembers: List[GroupMemberPrivilege] = ()
    childGroupMap: GroupChildGroup = None
    sd: ServiceData = None


@dataclass
class AddOrRemoveProjectMemberInput(TcBaseObj):
    """
    The structure holds the TC_Project object and the nodes to add or remove.
    
    :var project: The TC_Project object which the members are added or removed.
    :var gms: The GroupMember objects to be added or removed.
    :var groups: The Group objects to be added or removed.
    :var groupRoles: The GroupRole nodes to have the GroupMember objects under it added to or removed from the Project
    Team.
    :var addOrRemove: Indicates if the members need to be added or removed from the TC_Project. If true, the members in
    the input are added to the project. If false, the members in the input are removed from the project.
    """
    project: TC_Project = None
    gms: List[GroupMember] = ()
    groups: List[Group] = ()
    groupRoles: List[GroupRoleNode] = ()
    addOrRemove: bool = False


"""
The map for the Group object to its child GroupNodes.
"""
GroupChildGroup = Dict[Group, List[GroupNodeWithChildren]]



from __future__ import annotations

from tcsoa.gen.Internal.Gdis._2006_03.Reporting import PrecalRoutineInfoResponse
from tcsoa.utils import TcService


class ReportingService(TcService):

    @classmethod
    def getPrecalRoutineInfo(cls, routineId: str, routineRev: str, fromDate: str, toDate: str, specSetCode: str) -> PrecalRoutineInfoResponse:
        """
        Get the percentage on target pre calculations for a given routine, date range and spec set code
        
        Exceptions:
        >- When not able to get pre-calculations data for given input
        
        """
        return cls.execute_soa_method(
            method_name='getPrecalRoutineInfo',
            library='Internal-Gdis',
            service_date='2006_03',
            service_name='Reporting',
            params={'routineId': routineId, 'routineRev': routineRev, 'fromDate': fromDate, 'toDate': toDate, 'specSetCode': specSetCode},
            response_cls=PrecalRoutineInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.Workflow._2014_06.Workflow import SetActiveSurrogateInputInfo, PerformActionInputInfo, ApplySignatureInput, SurrogateInput
from tcsoa.gen.BusinessObjects import BusinessObject, User, EPMTask
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from datetime import datetime


class WorkflowService(TcService):

    @classmethod
    def performAction3(cls, input: List[PerformActionInputInfo]) -> ServiceData:
        """
        This operation performs an action on an EPMTask or Signoff. This operation also allows user to digitally sign
        the target objects in a workflow.
         
        Note:
        The Teamcenter session should be configured with HTTPS as the password in the input structure is not encrypted.
        
        Following actions are supported on EPMTask.
        
        - Assign
        - Start
        - Complete
        - Skip
        - Suspend
        - Resume
        - Undo
        - Perform
        - Approve
        - Reject
        - Promote
        - Demote
        - Claim
        
        
        
        Following action is supported on Signoff.
        
        - Assign
        
        
        This action will delegate a Signoff to a different GroupMember or ResourcePool and records the comments
        supplied while performing delegate operation in the audit log. User may use workflowService.getResourcePool
        method To get the ResourcePool business object from only group or only role or both.
        
        Use cases:
        Use Case 1: User can perform a workflow task.
        Description: User can perform an EPMTask from worklist folder in rich or thin client. Similarly EPMTask can be
        performed or signed-off using office client as well. This operation can be used to perform or signoff the
        workflow tasks.
        
        Use Case 2: User can assign or delegate a Signoff to a different GroupMember or ResourcePool and provide
        optional comments while performing delegate operation.
        Description: When a user delegates a Signoff to a different GroupMember or ResourcePool,  the comments provided
        while performing delegate operation gets recorded in the audit log.
        """
        return cls.execute_soa_method(
            method_name='performAction3',
            library='Workflow',
            service_date='2014_06',
            service_name='Workflow',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def performActionWithSignature(cls, task: EPMTask, action: str, comments: str, password: str, supportingValue: str, supportingObject: BusinessObject, signatures: List[ApplySignatureInput]) -> ServiceData:
        """
        This operation performs an action on workflow task. The following actions are supported.
        
        - Assign
        - Start
        - Complete
        - Skip
        - Suspend
        - Resume
        - Undo
        - Perform
        - Approve
        - Reject
        - Promote
        - Demote
        - Claim
        
        
        
        Use cases:
        User can perform a workflow task from worklist folder in rich or thin client. Similarly, workflow tasks
        can be performed or signed-off using office client as well.This operation can be used to perfrom or sign-off
        the workflow tasks.
        """
        return cls.execute_soa_method(
            method_name='performActionWithSignature',
            library='Workflow',
            service_date='2014_06',
            service_name='Workflow',
            params={'task': task, 'action': action, 'comments': comments, 'password': password, 'supportingValue': supportingValue, 'supportingObject': supportingObject, 'signatures': signatures},
            response_cls=ServiceData,
        )

    @classmethod
    def setActiveSurrogate(cls, input: List[SetActiveSurrogateInputInfo]) -> ServiceData:
        """
        This operation sets/unsets the logged in user as an active surrogate on a given EPMTask or Signoff. It
        transfers checkout of the target object(s) to/from the logged in user from/to the original user.
        
        Use cases:
        Use Case 1: User can Stand In for an EPMTask or Signoff with Transfer Check-Out(s).
        Description: When a user is set as a surrogate and wants to perform received task in the original user's inbox
        while allowing the original user to retain control, user can Stand-In for the EPMTask or Signoff. The transfer
        of checkout of the target object(s) from the original user is done when the boolean input parameter
        transferCheckouts is set to true. 
        
        Use Case 2: User is released from EPMTask or Signoff with Transfer Check-Out(s).
        Description: Releases current user as an active surrogate for a EPMTask or Signoff. The transfer of checkout of
        the target object(s) from the current user to the original user is done when the boolean input parameter
        transferCheckouts is set to true.
        
        Use Case 3: User can Stand-In for an EPMTask or Signoff without Transfer Check-Out(s).
        Description: When a user is set as a surrogate and wants to perform the task in the original user's inbox while
        allowing the original user to retain control, user can Stand-In for the EPMTask or Signoff. The target
        object(s) will remain checked out to the original user if they were checked out before the Stand-In operation
        was performed.
        
        Use Case 4: User is released for EPMTask or Signoff without Transfer Check-Out(s).
        Description: Releases current user as an active surrogate for an EPMTask or Signoff. The target object(s) will
        remain checked out to the current user if the transfer of checkout(s) was done while Stand-In operation was
        performed.
        """
        return cls.execute_soa_method(
            method_name='setActiveSurrogate',
            library='Workflow',
            service_date='2014_06',
            service_name='Workflow',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def setOutOfOffice(cls, fromResource: User, toResource: BusinessObject, startDate: datetime, endDate: datetime) -> ServiceData:
        """
        This operation sets/unsets Out of Office resource for a given user. The Out of Office resource can be a User or
        a ResourcePool. The Out of Office setting is effective for the specified date range. 
        
        Use cases:
        Use Case 1: User sets Out of Office
        Description: When a user is going to be out of the office and wants to have someone receive the tasks during
        his/her absence then user sets Out of Office. If the end date is not provided then all the future tasks will be
        delegated indefinitely. 
        
        Use Case 2: System Administrator sets Out of Office for another user
        Description: When a user is out of the office but does not have Out of Office setting then System Administrator
        can set Out of Office for the user. Also, group administrators will only be able to set the Out of Office for
        users within their group.
        
        Use Case 3:  User modifies Out of Office settings
        Description: When a user has set Out of Office and during this period if the Out of Office resource is also
        going to be out of the office then System Administrator can modify the Out of Office settings of the user 
        
        Use Case 4:  User unsets Out of Office settings
        Description: When a user who has set Out of Office is back before the specified end date then he/she can unset
        the current Out of Office setting. The startDate and endDate parameters will be set to null and the toResource
        paramenter will be set to an empty object.
        """
        return cls.execute_soa_method(
            method_name='setOutOfOffice',
            library='Workflow',
            service_date='2014_06',
            service_name='Workflow',
            params={'fromResource': fromResource, 'toResource': toResource, 'startDate': startDate, 'endDate': endDate},
            response_cls=ServiceData,
        )

    @classmethod
    def setSurrogate(cls, requests: List[SurrogateInput]) -> ServiceData:
        """
        This operation sets or unsets the surrogate resource for a given User effective for a given date range.
        """
        return cls.execute_soa_method(
            method_name='setSurrogate',
            library='Workflow',
            service_date='2014_06',
            service_name='Workflow',
            params={'requests': requests},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class RelateInfoIn(TcBaseObj):
    """
    Information to perform relate operation
    
    :var target: parent object to which the created object will be related. This value will be ignored if relate is
    false. If value is null and relate is true, then a default target will be used based on WsoInsertNoSelectionsPref.
    :var propertyName: Name of the property with which the created object will be related to the input target object if
    defined or the default one.This value will be ignored if relate is false. If value is empty and relate is true,
    then default relation will be used.
    :var relate: A relation is created only if this value is true.
    """
    target: BusinessObject = None
    propertyName: str = ''
    relate: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FeatureAssignmentStruct(TcBaseObj):
    """
    This is a structure of manufacturing feature objects, its corresponding occurrence chain, operation to which
    manufacturing feature objects are assigned and the assignment type.
    
    :var featureObject: Manufacutring feature objects, can be WeldPoint or ArcWeld.
    :var occChain: Occurrence chain of the manufacturing feature object.
    :var operation: Operation under which manufacturing feature object is assigned.
    :var assignedType: The type assignment of the manufacturing feature object i.e."Assigned", "UnAssigned", or
    "Invalid".
    Assigned means manufacturing feature object is connected to some parts in sourceContexts, it was not present in the
    targetContexts but its connected part was consumed under it. It is now assigned under the targetContext. UnAssigned
    means if the cycleTime property of process station is considered and it is less than the weldTime property of
    manufacturing feature object, the manufacturing feature is not assigned under the targetContexts. Invalid means
    manfacturing feature was earlier connected to some items in sourceContexts but now its connected item is now not
    present under targetContexts.
    """
    featureObject: BusinessObject = None
    occChain: str = ''
    operation: BusinessObject = None
    assignedType: str = ''


@dataclass
class AutomaticMFGFeaturesAssignmentInputInfo(TcBaseObj):
    """
    List of source and target BOMLine objects and assignment related information to be considered for automatic
    allocation of manufacturing features.
    
    :var sourceContexts: List of BOMLine objects under which the manufacturing features to be assigned are present.
    :var targetContexts: The list of process station BOMLine objects in BOPWindow under which the manufacturing
    features will be assigned.
    :var considerCycleTime: If true, the cycle time of process station will be considered while assigment.
    :var matchType: The type of search to perform on the structure. Values may be "NotApplicableMatch", "FullMatch",
    "PartialMatch" or "NoMatch".
    """
    sourceContexts: List[BusinessObject] = ()
    targetContexts: List[BusinessObject] = ()
    considerCycleTime: bool = False
    matchType: str = ''


@dataclass
class AutomaticMFGFeaturesAssignmentResponse(TcBaseObj):
    """
    It is a response for automaticMFGFeaturesAssignment operation.
    
    :var assignmentList: Structure with member as Operation or Process and the list of FeatureAssignmentStruct objects.
    :var logFileTicket: Log file ticket to generate the reports.
    :var logFileName: The name of log file which contain the reports.
    :var serviceData: The ServiceData contains partial errors if any.
    """
    assignmentList: List[StationAssignmentStruct] = ()
    logFileTicket: str = ''
    logFileName: str = ''
    serviceData: ServiceData = None


@dataclass
class StationAssignmentStruct(TcBaseObj):
    """
    This structure contains the station and the list of FeatureAssignmentStruct objects.
    
    :var parentObject: Process station where Operation consuming manufacturing feature objects are assigned.
    :var featureAssignmentList: A list of objects containing information related to assignment of manufacturing feature
    object.
    """
    parentObject: BusinessObject = None
    featureAssignmentList: List[FeatureAssignmentStruct] = ()



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.BusinessObjects import ScheduleTask, ResourcePool
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignedTasksInfo(TcBaseObj):
    """
    Contains information about the assigned ScheduleTask objects  to the logged in User.
    
    :var state: State of the SchduleTask. Valid values are: "not_started","in_progress","complete", "closed" and
    "aborted".
    :var status: Status of the SchduleTask. Valid values are: "not_started", "in_progress", "needs_attention", "late",
    "completed", "abandoned" and "aborted".
    :var loadedTasks: A list of loaded Schedule Task objects assigned to the logged in User.
    :var foundTasks: The foundTasks will be populated if loadOptions  is set  to "populateFoundTasksUIDs = "true" in
    LoadAssignedTaskInfo structure. If "false", foundTasks will not be populated.
    :var nFoundTasks: The number of Schedule Task objects found for the logged in User.
    """
    state: str = ''
    status: str = ''
    loadedTasks: List[ScheduleTask] = ()
    foundTasks: List[str] = ()
    nFoundTasks: int = 0


@dataclass
class LoadAssignedTasksInfo(TcBaseObj):
    """
    Information to fetch the ScheduleTask objects assigned to the logged in User.
    
    :var state: State of the SchduleTask. Valid values are: "not_started","in_progress","complete", "closed" and
    "aborted".
    :var status: Status of the SchduleTask. Valid values are: "not_started", "in_progress", "needs_attention", "late",
    "completed", "abandoned" and "aborted".
    :var startIndex: Pagination value. User can specify the the start index of the next chunk of objects to return. For
    example: This operation loads 100 ScheduleTask objects. User want to get first 50 loaded objects. So here the
    startIndex would be 0. For next chunk of loaded objects this input would be 51. Note Value for this input should be
    a positive integer. If the value is negative number then it is considered as 0.
    :var maxToLoad: The maximum number of found ScheduleTask objects to load. Note: Value for this input should be a
    positive integer. If the value is 0 or negative number then this input will be ignored and all the found
    ScheuleTask objects will be loaded.
    :var loadOptions: A map( sttring, string) of options for loading a ScheduleTask objects. Valid options (key :
    value) are:
    "populateFoundTasksUIDs" : "true" or "false".  If "true", foundTasks element in AssignedTasksInfo structure will be
    populated with the UIDs of all schedule tasks assigned to the logged in user. If "false", foundTasks will not be
    populated.
    """
    state: str = ''
    status: str = ''
    startIndex: int = 0
    maxToLoad: int = 0
    loadOptions: AssignedTasksLoadOptions = None


@dataclass
class LoadAssignedTasksResponse(TcBaseObj):
    """
    Response of the loadAssignedTasks operation, containing information about the assigned ScheduleTask objects  to the
    logged in User.
    
    :var assignedTasks: A list of assigned Schedule Task objects to the logged in User.
    :var serviceData: The service data.
    """
    assignedTasks: List[AssignedTasksInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ClaimAssignmentData(TcBaseObj):
    """
    The claimAssignmentData structure holds information needed to claim the assignments.
    
    :var scheduleTask: The ScheduleTask to be assigned to the logged in user.
    :var assignedResourcePool: The ResourcePool assigned to the ScheduleTask. The logged in user must be a member of
    the ResourcePool object.
    """
    scheduleTask: ScheduleTask = None
    assignedResourcePool: ResourcePool = None


"""
Map (string, string) containg a key value pair of string type.
"""
AssignedTasksLoadOptions = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetCAEPropertyComparisonDetailsResponse(TcBaseObj):
    """
    Data structure to capture the response of 'getCAEPropertyComparisonDetails' method. This has property comparison
    details for selected BOMLine object and 'serviceData' to capture the partial errors.
    
    :var result: Detail results of the comparison for the selected BOMLine object. This is an XML document that
    conforms to the http://www.ugs.com/Schemas/SMENode schema.
    :var serviceData: 'serviceData' member to capture any partial errors.
    """
    result: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2016_03.Finder import SearchFilterField, ObjectsGroupedByProperty
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ColumnConfig2(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information, and the operation type used to finalize the columns.
    
    :var columnConfigId: The unique identifier of the column configuration.
    :var operationType: The operation that was used to finalize the columns to be returned back. Supported values are:
    "Intersection", "Union" and "Configured".
    :var columns: List of column details.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo2] = ()


@dataclass
class SearchFilter3(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Supported values are "StringFilter", "DateFilter",
    "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var colorValue: The color of the Search Filter.
    :var stringDisplayValue: The display value for a string filter. This field is applicable only if the
    "searchFilterType" field is set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter"
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter"
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the searchFilterType
    is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: If true, the Search Filter was previously selected; otherwise, the Search Filter was not previously
    selected.
    :var startEndRange: The interval used to generate a range facets. E.g for Date range facets, +1DAY, +1WEEK, +1MONTH
    , +1YEAR are acceptable values. Note: Currently this value is not being used. To be supported in the future.
    """
    searchFilterType: str = ''
    stringValue: str = ''
    colorValue: str = ''
    stringDisplayValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchResponse3(TcBaseObj):
    """
    A service response structure containing search results and column configurations.
    
    :var searchResults: List of ViewModel business objects obtained after performing a search.
    :var totalFound: Total number of ViewModel business objects found.
    :var propDescriptors: List of ViewModelPropertyDescriptors required to render the object properties in stylesheet.
    :var totalLoaded: Total number of ViewModel business objects loaded.
    :var searchFilterMap3: A map (string, list of SearchFilter2) containing the list of search filters for each search
    filter field based on the search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var objectsGroupedByProperty: The structure containing an internal property name and a map of objects to the
    property group id. It also contains a list of unmatched objects which do not match any group.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: The service data object.
    :var endIndex: Cursor end position for the results returned so far. This value, 'endIndex', correlates with
    'startindex' in SearchInput.
    """
    searchResults: List[ViewModelObject] = ()
    totalFound: int = 0
    propDescriptors: List[ViewModelPropertyDescriptor] = ()
    totalLoaded: int = 0
    searchFilterMap3: SearchFilterMap3 = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    objectsGroupedByProperty: ObjectsGroupedByProperty = None
    columnConfig: ColumnConfig2 = None
    serviceData: ServiceData = None
    endIndex: int = 0


@dataclass
class ViewModelObject(TcBaseObj):
    """
    Service operation output structure to store the Teamcenter BusinessObject and its associated View model properties
    to be displayed in the view.
    
    :var modelObject: Teamcenter BusinessObject that was searched for and to be rendered in the view.
    :var viewModelProperties: List of ViewModelProperty structures
    """
    modelObject: BusinessObject = None
    viewModelProperties: List[ViewModelProperty] = ()


@dataclass
class ViewModelProperty(TcBaseObj):
    """
    ViewModelProperty represents a property that is displayed in the view, it can be simple property, runtime property,
    compound property or dynamic compound property.
    
    :var propertyName: Property name
    :var values: Real value for the property, the values are converted to strings for other value types such as date,
    double, Boolean or referenced objects.
    :var displayValues: Display values for the property
    :var intermediateObjectUids: List of UID's of intermediate objects across the property traversal path.
    :var srcObjLsd: Last saved date of source object
    :var isModifiable: The flag that indicates if the property is modifiable
    """
    propertyName: str = ''
    values: List[str] = ()
    displayValues: List[str] = ()
    intermediateObjectUids: List[str] = ()
    srcObjLsd: datetime = None
    isModifiable: bool = False


@dataclass
class ViewModelPropertyDescriptor(TcBaseObj):
    """
    Represents the property descriptor details for a ViewModelProperty, the definition of ViewModelPropertyDescriptor
    is taken from the property descriptor of the source object property.
    
    :var srcObjectTypeName: Business object type name of the source object on which the property is defined.
    :var propertyName: Name of the property, it includes the traversal path for dynamic compound property. It is same
    as the propertyName in ViewModelProperty.
    :var valueType: Indicates the value type for the property, valid values are: 
     value - type 
     0 . PROP_untyped -  No Value type  
     1 . PROP_char - Value is a single character * 
     2 . PROP_date - Value is a date  
     3 . PROP_double - Value is a double  
     4 . PROP_float - Value is a float  
     5 . PROP_int -  Value is an integer  
     6 . PROP_logical -  Value is a logical  
     7 . PROP_short - Value is a short  
     8 . PROP_string - Value is a character string  
     9 . PROP_typed_reference - Value is a typed reference  
    10 . PROP_untyped_reference - Value is an untyped reference  
    11 . PROP_external_reference - Value is an external reference  
    12 . PROP_note - Value is a note  
    13 . PROP_typed_relation - Value is a typed relation  
    14 . PROP_untyped_relation - Value is an untyped relation etc. 
    This is from the source object property descriptor.
    :var isArray: Indicates if the property is a single value property or an array, this is from the source object
    property descriptor.
    :var propConstants: Map ( string, /string ) consisting of propertyConstant name and value pairs, this is from the
    source object property descriptor.
    :var displayName: Display name of the property, if columnDisplayNameKey is defined in column config, the display
    name is the values corresponds to it in text server; otherwise, the displayName is the source object property
    display name.
    :var lovCategory: Indicates the LOV category, if the property is attached to LOV.
    :var lov: LOV reference associated with the property, this is from the source object property descriptor.
    :var maxArraySize: Maximum number of elements allowed in case of array property, this is from the source object
    property descriptor.
    :var maxLength: Max allowed length for the property if property is string property.
    :var propertyType: Indicates the value type for the property, valid values are:
    value - type 
    0 . PROP_unknown  - Property type is Unknown  
    1 . PROP_attribute  - Based on a POM Attribute (int, string, ...)  
    2 . PROP_reference -  Based on a POM Reference  
    3 . PROP_relation - Based on an ImanRelation  
    4 . PROP_compound - Based on a property from another Type  
    5 . PROP_runtime - Based on a computed value  
    6 . PROP_operationinput - Based on the source property on a BO  
    This is from the source object property descriptor.
    """
    srcObjectTypeName: str = ''
    propertyName: str = ''
    valueType: int = 0
    isArray: bool = False
    propConstants: ViewModelPropertyConstantsMap = None
    displayName: str = ''
    lovCategory: int = 0
    lov: BusinessObject = None
    maxArraySize: int = 0
    maxLength: int = 0
    propertyType: int = 0


@dataclass
class ColumnDefInfo2(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client, pixel width of the column, a flag indicating if the column should be hidden and the column
    sort order.
    
    :var displayName: The display name for the value displayed in the column header.
    :var typeName: The business object type for the value displayed in the column. This can be any valid Teamcenter
    business object type.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: If true, the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort. Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted. Supported values are: "Ascending" and "Descending". This value will
    be empty if the column is not marked for sorting.
    """
    displayName: str = ''
    typeName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap3 = Dict[str, List[SearchFilter3]]


"""
Map to store the Property constant name and value pair
"""
ViewModelPropertyConstantsMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetPrecedenceConstraintPathsInputs(TcBaseObj):
    """
    Input structure for the getPrecedenceConstraintPaths operation.
    
    :var startNode: The process/operation line at the start of the precedence chain.
    This member can hold BOs of type Mfg0BvrProcess and Mfg0BvrOperation.
    :var endNode: The operation/process line at the end of the precedence chain
    This member can hold BOs of type Mfg0BvrProcess and Mfg0BvrOperation.
    """
    startNode: BusinessObject = None
    endNode: BusinessObject = None


@dataclass
class GetPrecedenceConstraintPathsResponse(TcBaseObj):
    """
    Response structure for the getPrecedenceConstraintPaths operation.
    
    :var results: A list of result structures that contain the paths for each start and end node pair specfied in the
    input structures.
    :var serviceData: The ServiceData object for the matching GetPrecedenceConstraintPathsInputs structure.
    """
    results: List[GetPrecedenceConstraintPathsResult] = ()
    serviceData: ServiceData = None


@dataclass
class GetPrecedenceConstraintPathsResult(TcBaseObj):
    """
    The result structure for the getPrecedenceConstraintPaths operation.
    
    :var paths: The list of paths that connect the respective start and end node.
    """
    paths: List[PrecedenceConstraintPath] = ()


@dataclass
class GetPrecedenceConstraintsIn(TcBaseObj):
    """
    The input structure for the getPrecedenceConstraints service operation
    
    :var predecessorLevel: The integer indicating the number of predecessor levels to be processed in the precedence
    chain
    :var successorLevel: The integer indicating the number of successor levels to be processed in the precedence chain
    :var inputObject: The input object (Mfg0BvrProcess / Mfg0BvrOperation)
    """
    predecessorLevel: int = 0
    successorLevel: int = 0
    inputObject: BusinessObject = None


@dataclass
class GetPrecedenceConstraintsResponse(TcBaseObj):
    """
    The response structure of the getPrecedenceConstraints service operation. Contains the predecessor and successor
    maps.
    
    :var predecessorMap: The map of objects to their predecessors
    :var successorMap: The map of objects to their successors
    :var serviceData: The ServiceData structure
    """
    predecessorMap: AdjacentObjectsMap = None
    successorMap: AdjacentObjectsMap = None
    serviceData: ServiceData = None


@dataclass
class PrecedenceConstraintPath(TcBaseObj):
    """
    Defines a specific path for the GetPrecedenceConstraintPaths API
    
    :var components: A list of structures that hold the nodes and constraints that make up the path to the end node.
    :var endNode: The end node itself. The Business Object  is of type Mfg0BvrProcess or Mfg0BvrOperation.
    """
    components: List[PrecedenceConstraintPathComponent] = ()
    endNode: BusinessObject = None


@dataclass
class PrecedenceConstraintPathComponent(TcBaseObj):
    """
    A component of a constraint path for structure PrecedenceConstraintPath
    
    :var predecessor: A node which is part of the path (type Mfg0BvrProcess or Mfg0BvrOperation).
    :var constraint: The constraint that connects the node of this component structure to the next, or, in case this is
    the last component in the path, to the end node. Note that the predecessor of the constraint is different from the
    predecessor member in case the constraint is inherited.The same applies to the successor of the constraint: it does
    not necessarily have to be the successor in the path. This member is of type Mfg0BvrPrecedenceConstraint.
    :var constraintIsInherited: Indicates whether the constraint is directly connected to the predecessor and successor
    nodes in the path or whether it has been inherited from a parent node:
    """
    predecessor: BusinessObject = None
    constraint: BusinessObject = None
    constraintIsInherited: bool = False


@dataclass
class ValidateConstraintConsistencyError(TcBaseObj):
    """
    Describes a problem detected by the validateConstraintConsistency operation.
    
    :var constraintType: The type of the constraint that is violated. This is the ImanType of the runtime constraint
    object, for example Mfg0BvrPrecedenceConstraint for precedence constraints.
    :var message: A localized message explaining the detected inconsistency.
    :var objects: The list of operations/processes that participate in the constraints that are inconsistent. The list
    contains business objects of type Mfg0BvrOperation and Mfg0BvrProcess.
    :var constraints: The constraint objects which are inconsistent.These objects are of type Mfg0BvrConstraint.
    """
    constraintType: BusinessObject = None
    message: str = ''
    objects: List[BusinessObject] = ()
    constraints: List[BusinessObject] = ()


@dataclass
class ValidateConstraintConsistencyInputs(TcBaseObj):
    """
    Input structure for the validateConstraintConsistency operation.
    
    :var constraintTypes: The constraint types to check. Each entry in this vector is of type ImanType; it corresponds
    to the type of the runtime object for the constraint (e.g. Mfg0BvrPrecedenceConstraint for precedence constraints
    or Mfg0BvrPrecedenceConstraint for group constraints; customer defined derived types are accepted as well). If the
    vector is empty, all constraints are checked.
    :var rootNodes: The list of BOMLines from the BOP window that define the root nodes of the sub structures to check.
     The objects must be of type Mfg0BvrProcess or Mfg0BvrOperation. All lines must be associated with the same window.
    """
    constraintTypes: List[BusinessObject] = ()
    rootNodes: List[BusinessObject] = ()


@dataclass
class ValidateConstraintConsistencyResponse(TcBaseObj):
    """
    Response structure of the validateConstraintConsistencyResponse operation.
    
    :var results: The list of result structures that holds one entry for each input structure.
    :var serviceData: The ServiceData object for this request.
    """
    results: List[ValidateConstraintConsistencyResult] = ()
    serviceData: ServiceData = None


@dataclass
class ValidateConstraintConsistencyResult(TcBaseObj):
    """
    Result structure for the validateConstraintConsistency operation.
    
    :var errors: The  list of validation errors, if any.
    """
    errors: List[ValidateConstraintConsistencyError] = ()


@dataclass
class ValidateProcessAreaAssignmentsError(TcBaseObj):
    """
    A structure that contains information about an problem detected in the ValidateProcessAreaAssignments operation.
    In the case of a precedence constraint violation, the objectsOfContextOfConstraint vector will contain the
    predecessor and the sucessor operation or process whose order is conflicting with the order of the process areas
    they are assigned to, if a Product BOP is given. The objectsOfContextToValidate member will contain their
    respective counterparts in the Plant BOP, and processAreas will contain the associated process areas. In case of a
    precedence constraint violations the constraints vector will be empty, since the constraints themselves are
    meaningless without the intermittent operations/processes. In order to obtain the paths of processes/operations and
    constraints that define the precedence of the start and end node, the GetPrecedenceConstraintPathsAPI may be used.
    Currently one possible type of error may arise when precedence constraints are violated. The format of the error
    message is similar to the following:
    <Node A> is assigned to <Process Area X> while its successor <Node B> is assigned to <Process Area Y>  which is a
    predecessor of  <Process Area X>.
    
    :var constraintType: The type of the constraint that is violated. Corresponds to the ImanType of the constraint
    object. See also the constraintTypes member of struct ValidateProcessAreaAssignmentsInputs.
    :var message: A localized message explaining the validation error.
    :var objectsOfContextOfConstraint: The list of operations and/or processes that violate the constraints. These
    objects belong to the contexts passed via the contextOfConstraints parameter i.e. the Product BOPs. The vector will
    be empty if the contextOfConstraints vector is empty. The list contains business objects of type Mfg0BvrOperation
    and Mfg0BvrProcess.
    :var objectsOfContextToValidate: The list of assigned nodes from the contextToValidate parameter. In case only a
    single Product BOP exists, the vector will hold the same number of objects as the objectsOfContextOfConstraint
    member. The business objects are of type Mfg0BvrOperation and Mfg0BvrProcess.
    :var processAreas: The list of process areas associated with the Product BOP objects. The objects are of type
    Mfg0BvrProcessArea and belong to the context passed in contextToValidate.
    :var constraints: The constraint objects which are violated. These objects are of type Mfg0BvrConstraint. Note that
    depending on the type of violation the constraints list may be empty. This is the case for example for precedence
    constraints.
    """
    constraintType: BusinessObject = None
    message: str = ''
    objectsOfContextOfConstraint: List[BusinessObject] = ()
    objectsOfContextToValidate: List[BusinessObject] = ()
    processAreas: List[BusinessObject] = ()
    constraints: List[BusinessObject] = ()


@dataclass
class ValidateProcessAreaAssignmentsInputs(TcBaseObj):
    """
    Input structure for the validateProcessAreaAssignments operation.
    
    :var constraintTypes: The constraint types to check. Each entry in this vector is of type ImanType; it corresponds
    to the type of the runtime object for the constraint (e.g. Mfg0BvrPrecedenceConstraint for precedence constraints
    or Mfg0BvrGroupConstraint for group constraints). The types vector may also contain customized types derived from
    the base constraint types. If the vector is empty, all constraints are checked.
    :var contextToValidate: The context of the process area structure. This is the top line of the window that holds
    the Plant BOP structure (the object is of type Mfg0BvrPlantBOP).
    :var contextOfConstraints: A list of contexts that describes the processes and operations whose assignments are to
    be validated. If empty it is assumed that the context of the process area structure is to be used. Currently only
    one single is accepted that represents the topline of the Product BOP structure (this object must be of type
    Mfg0BvrProductBOP).
    :var rootProcessAreas: The list of BOMLines from the Plant BOP window that define the substructure whose
    allocations are examined If the list is empty, the whole Plant BOP loaded into the Plant BOP window will be
    checked. The Business Objects are of type Mfg0BvrProcessArea.
    :var maxErrors: The maximum number of errors to return. Can be set to 0 in order not to impose any limit. Otherwise
    the value must be greater than 0.
    """
    constraintTypes: List[BusinessObject] = ()
    contextToValidate: BusinessObject = None
    contextOfConstraints: List[BusinessObject] = ()
    rootProcessAreas: List[BusinessObject] = ()
    maxErrors: int = 0


@dataclass
class ValidateProcessAreaAssignmentsResponse(TcBaseObj):
    """
    Response structure for the validateProcessAreaAssignments operation.
    
    :var results: The list of result structures that holds one entry for each input structure.
    :var serviceData: The ServiceData object for this request.
    """
    results: List[ValidateProcessAreaAssignmentsResult] = ()
    serviceData: ServiceData = None


@dataclass
class ValidateProcessAreaAssignmentsResult(TcBaseObj):
    """
    Result structure for the validateProcessAreaAssignments operation.
    
    :var errors: The  list of validation errors, if any.
    """
    errors: List[ValidateProcessAreaAssignmentsError] = ()


@dataclass
class AdjacentObjectInfo(TcBaseObj):
    """
    The structure representing an adjacent object (either the predecessor or the successor) along with the constraint
    object
    
    :var constraintObject: The constraint object (Mfg0BvrPrecedenceConstraint)
    :var adjacentObject: The preceding or succeeding object (Mfg0BvrProcess / Mfg0BvrOperation)
    """
    constraintObject: BusinessObject = None
    adjacentObject: BusinessObject = None


"""
The map containing a BusinessObject as key and its list of adjacent objects (predecessor/successor) as the value
"""
AdjacentObjectsMap = Dict[BusinessObject, List[AdjacentObjectInfo]]



from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2015_10.ConfiguratorManagement import ConfigExpressionTerm, BusinessObjectConfigExpression
from tcsoa.gen.BusinessObjects import Cfg0AbsFamily, Cfg0ConfiguratorPerspective
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AvailabilityInput(TcBaseObj):
    """
    It is the object containing input information required to compute the availability.
    
    :var criteriaExpression: A list of structures each relating a business object to a set of configuration expressions.
    For the details of the structure BusinessObjectConfigExpression please refer to
    Cfg0::Soa::Internal::Configurator::_2015_10::ConfiguratorManagement:: BusinessObjectConfigExpression
    :var familiesToTest: The list of configuration families (Cfg0AbsFamily) for which available features are requested.
    :var valuesToTest: The list of Configuration Expression Terms (ConfigExpressionTerm) whose availability is to be
    calculated.
    :var perspective: Configurator perspective details.
    Property cfg0ProductItems of perspective must contain reference to a single Configurator context item and property
    cfg0RevisonRule must be set to a RevisionRule
    :var applyConstraints: Defines the mode to apply the constraints. The value is a bitwise OR the following values: 
    0: disable configurator constraint evaluation 
    1: enable configurator constraint evaluation 
    512:  get available features for each family individually after resetting the config expression for the inquired
    family
    2048: enable Availability Constraint evaluation.
    """
    criteriaExpression: List[BusinessObjectConfigExpression] = ()
    familiesToTest: List[Cfg0AbsFamily] = ()
    valuesToTest: List[ConfigExpressionTerm] = ()
    perspective: Cfg0ConfiguratorPerspective = None
    applyConstraints: int = 0


@dataclass
class AvailabilityOutput(TcBaseObj):
    """
    An object containing input information required to represent avialability of features for models.
    
    :var availabilityStruct: A list of list containing AvailableFamilyVariabilityStruct. This represents availability
    records for all the models in the input. The entry corresponds to the input by index.
    :var serviceData: The service data for errors and returned objects.
    """
    availabilityStruct: List[AvailableFamilyVariabilityStruct] = ()
    serviceData: ServiceData = None


@dataclass
class AvailableFamilyVariability(TcBaseObj):
    """
    Defines the availability status of option values from the given family input for the operation
    getOptionValueAvailability.
    
    :var availability: Specifies the list of configuration expression terms referencing available features from the
    requested family.
    :var availibilityStatus: A string representing the availability status of the feature. Presently three values are
    supported
    - Available
    - Not Available
    - Not Editable
    
    """
    availability: List[ConfigExpressionTerm] = ()
    availibilityStatus: List[str] = ()


@dataclass
class AvailableFamilyVariabilityStruct(TcBaseObj):
    """
    Represents availability for features for a single model.
    
    :var availabilityExpressions: List of family variability, each element represents variability of features form a
    single family
    """
    availabilityExpressions: List[AvailableFamilyVariability] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Notification._2014_10.SubscriptionManagement import GetSubscriptionConditionsResponse
from tcsoa.utils import TcService


class SubscriptionManagementService(TcService):

    @classmethod
    def getSubscriptionConditions(cls) -> GetSubscriptionConditionsResponse:
        """
        This operation returns all the Conditions that can be used with Subscriptions.  For a Condition to be used with
        a Subscription, its signature must contain a POM_object and a UserSession.
        
        The results of this operation will be used with the Subscription Manager UI that will allow the user to select
        a Condition for use with a Subscription.
        
        Use cases:
        The client is searching for a list of all Conditions that have the correct signature to be used with
        Subscriptions.  This list will be displayed to the user from the Subscription Manager UI.
        """
        return cls.execute_soa_method(
            method_name='getSubscriptionConditions',
            library='Internal-Notification',
            service_date='2014_10',
            service_name='SubscriptionManagement',
            params={},
            response_cls=GetSubscriptionConditionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GlobalAlternateList(TcBaseObj):
    """
    Holds a vector of global alternates, to be used by GlobalAlternateResponse Struct
    
    :var globalAlternates: A list of global alternates for the selected item
    """
    globalAlternates: List[Item] = ()


@dataclass
class GlobalAlternateListInput(TcBaseObj):
    """
    The GlobalAlternateListInput struct represents all of the data necessary to
    add and remove a list of global alternates.
    
    :var item: the selected item
    :var gAltItems: a list of global alternates to be added or removed
    """
    item: Item = None
    gAltItems: List[Item] = ()


@dataclass
class GlobalAlternateResponse(TcBaseObj):
    """
    Holds the response for listing, adding and removing global alternates
    
    :var globalAlternateLists: a vector of updated global alternate lists
    :var serviceData: Exceptions from internal processing returned as PartialErrors
    """
    globalAlternateLists: List[GlobalAlternateList] = ()
    serviceData: ServiceData = None


@dataclass
class PreferredGlobalAlternateInput(TcBaseObj):
    """
    The PreferredGlobalAlternateInput struct represents all of the data necessary to
    set the preferred global alternate of the selected item.
    
    :var item: the selected item
    :var gAltItem: the item to be set as preferred global alternate
    """
    item: Item = None
    gAltItem: Item = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetLicenseBundlesResponse(TcBaseObj):
    """
    List of license bundle information, one for each unique license server. It represents the response data structure
    for the getLicenseBundles operation.
    
    :var licServerInfo: List of license server information.
    :var serviceData: Contains any partial errors.
    """
    licServerInfo: List[LicenseServerInfo] = ()
    serviceData: ServiceData = None


@dataclass
class LicenseBundleInfo(TcBaseObj):
    """
    This structure represents the details of a license bundle.
    
    :var licenseBundleName: The name of a license bundle. It is a string of upto 9 bytes in size.
    :var baseFeatureKey: The base license level feature key included in the license bundle. An empty string for this
    element indicates that there is no base license feature key in the license bundle.
    :var numberPurchased: The number of licenses purchased for the license bundle.
    :var expirationDate: The date after which the license bundle is not available for use.
    """
    licenseBundleName: str = ''
    baseFeatureKey: str = ''
    numberPurchased: int = 0
    expirationDate: datetime = None


@dataclass
class LicenseServerInfo(TcBaseObj):
    """
    This structure holds the name of the license server and a list of all license bundles associated with the license
    server. If there are no license bundles in a license server, then an empty list is returned.
    
    :var licenseServerName: The name of the license server for which bundles are fetched.
    :var licenseBundles: List of license bundles associated with this license server.
    """
    licenseServerName: str = ''
    licenseBundles: List[LicenseBundleInfo] = ()



from __future__ import annotations

from tcsoa.gen.Internal.DocumentManagement._2020_12.AttributeExchange import ProcessAttrExchangeConfigInput
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class AttributeExchangeService(TcService):

    @classmethod
    def processAttrExchangeConfigurations(cls, processAttrExchConfInputInfo: ProcessAttrExchangeConfigInput) -> ServiceData:
        """
        This operation adds, update or delete  Fnd0AttrExchangeConfig objects that are referenced by
        Fnd0LogicalObjectType.
        
        Use cases:
        This operation is invoked in the Exchange Configuration tab on an existing Logical Object Type.
        """
        return cls.execute_soa_method(
            method_name='processAttrExchangeConfigurations',
            library='Internal-DocumentManagement',
            service_date='2020_12',
            service_name='AttributeExchange',
            params={'processAttrExchConfInputInfo': processAttrExchConfInputInfo},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateArrangementsResponse(TcBaseObj):
    """
    GenerateArrangementsResponse struct
    
    :var fileTicket: The FMS ticket is used to get the generated PLMXML file.
    :var serviceData: The service data is used to report any partial failures.
    """
    fileTicket: str = ''
    serviceData: ServiceData = None



from tcsoa.gen.Internal_ActiveWorkspaceBom._2018_12.services import OccurrenceManagementService as imp0
from tcsoa.gen.Internal_ActiveWorkspaceBom._2019_06.services import OccurrenceManagementService as imp1
from tcsoa.gen.Internal_ActiveWorkspaceBom._2016_03.services import OccurrenceManagementService as imp2
from tcsoa.gen.Internal_ActiveWorkspaceBom._2018_05.services import OccurrenceManagementService as imp3
from tcsoa.gen.Internal_ActiveWorkspaceBom._2018_12.services import MarkupService as imp4
from tcsoa.gen.Internal_ActiveWorkspaceBom._2015_03.services import OccurrenceManagementService as imp5
from tcsoa.gen.Internal_ActiveWorkspaceBom._2018_12.services import CompareService as imp6
from tcsoa.gen.Internal_ActiveWorkspaceBom._2019_06.services import CompareService as imp7
from tcsoa.gen.Internal_ActiveWorkspaceBom._2017_06.services import OccurrenceManagementService as imp8
from tcsoa.gen.Internal_ActiveWorkspaceBom._2012_10.services import BOMIndexManagementService as imp9
from tcsoa.gen.Internal_ActiveWorkspaceBom._2019_12.services import OccurrenceConfigurationService as imp10
from tcsoa.gen.Internal_ActiveWorkspaceBom._2019_12.services import OccurrenceManagementService as imp11
from tcsoa.gen.Internal_ActiveWorkspaceBom._2015_07.services import OccurrenceManagementService as imp12
from tcsoa.gen.Internal_ActiveWorkspaceBom._2020_12.services import OccurrenceManagementService as imp13
from tcsoa.gen.Internal_ActiveWorkspaceBom._2012_10.services import OccurrenceManagementService as imp14
from tcsoa.gen.Internal_ActiveWorkspaceBom._2017_06.services import OccurrenceConfigurationService as imp15
from tcsoa.gen.Internal_ActiveWorkspaceBom._2017_12.services import OccurrenceManagementService as imp16
from tcsoa.gen.Internal_ActiveWorkspaceBom._2019_12.services import DataManagementService as imp17
from tcsoa.gen.Internal_ActiveWorkspaceBom._2015_10.services import OccurrenceManagementService as imp18
from tcsoa.gen.Internal_ActiveWorkspaceBom._2020_05.services import OccurrenceManagementService as imp19
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):
    addObject = imp0.addObject
    addObject2 = imp1.addObject2
    addOrRemoveOccurrenceEffectivities = imp2.addOrRemoveOccurrenceEffectivities
    addSubstitutes = imp3.addSubstitutes
    addToBookmark2 = imp2.addToBookmark2
    attachObjects = imp5.attachObjects
    cloneContent = imp1.cloneContent
    cloneContentAsync = imp1.cloneContentAsync
    createAndAddElementEffectivity = imp8.createAndAddElementEffectivity
    detachObjects = imp11.detachObjects
    duplicateAndReplace = imp3.duplicateAndReplace
    duplicateAndReplaceAsync = imp3.duplicateAndReplaceAsync
    editElementEffectivity = imp8.editElementEffectivity
    findMatchingFilters = imp12.findMatchingFilters
    findMatchingFilters2 = imp11.findMatchingFilters2
    getAllowedOccurrenceTypes = imp13.getAllowedOccurrenceTypes
    getChildOccurrences = imp14.getChildOccurrences
    getInfoForAddElement = imp14.getInfoForAddElement
    getInfoForAddElement2 = imp1.getInfoForAddElement2
    getInfoForAddElement3 = imp11.getInfoForAddElement3
    getInfoForAddToBookmark = imp14.getInfoForAddToBookmark
    getInfoForInsertLevel = imp13.getInfoForInsertLevel
    getNextChildOccurrences = imp14.getNextChildOccurrences
    getNextOccurrencesInProduct = imp14.getNextOccurrencesInProduct
    getOccurrences = imp0.getOccurrences
    getOccurrences2 = imp1.getOccurrences2
    getOccurrences3 = imp11.getOccurrences3
    getOccurrencesInProduct = imp14.getOccurrencesInProduct
    getPackedOccurrenceCSIDs = imp16.getPackedOccurrenceCSIDs
    getSubsetInfo2 = imp2.getSubsetInfo2
    getSubsetInfo3 = imp11.getSubsetInfo3
    insertLevel = imp14.insertLevel
    insertLevel2 = imp13.insertLevel2
    packSimilarElements = imp1.packSimilarElements
    preferSubstitute = imp3.preferSubstitute
    removeElements = imp3.removeElements
    removeInContextPropertyOverride = imp11.removeInContextPropertyOverride
    removeLevel = imp13.removeLevel
    removeSubstitutes = imp3.removeSubstitutes
    replaceElement = imp2.replaceElement
    resetUserWorkingContextState = imp13.resetUserWorkingContextState
    saveSelections = imp18.saveSelections
    saveUserWorkingContextState = imp8.saveUserWorkingContextState
    saveUserWorkingContextState2 = imp1.saveUserWorkingContextState2
    saveWorkingContext = imp19.saveWorkingContext
    updateContentBasedOnRevision = imp19.updateContentBasedOnRevision
    updateSavedBookmark = imp14.updateSavedBookmark
    updateWorkingContext = imp19.updateWorkingContext


class MarkupService(TcService):
    applyMarkup = imp4.applyMarkup
    cancelMarkup = imp4.cancelMarkup


class CompareService(TcService):
    compareContent2 = imp6.compareContent2
    compareContentAsync2 = imp7.compareContentAsync2
    getCompareOptions = imp6.getCompareOptions


class BOMIndexManagementService(TcService):
    createBOMIndexAdminData = imp9.createBOMIndexAdminData
    processBomIndex = imp9.processBomIndex


class OccurrenceConfigurationService(TcService):
    createOrUpdateClassicVariantRule = imp10.createOrUpdateClassicVariantRule
    getClassicVariants = imp10.getClassicVariants
    getConfigurationRules = imp15.getConfigurationRules
    getConfigurationRules2 = imp10.getConfigurationRules2


class DataManagementService(TcService):
    getViewModelForCreate = imp17.getViewModelForCreate



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.Reports._2008_12.CrfReports import TransientFileTicketInfo
from dataclasses import dataclass


@dataclass
class GeneratePrintReportsOutput(TcBaseObj):
    """
    A structure that contains information about the generated print reports.
    
    :var clientID: The unmodified client id from the input PrintReportsCriteria structure to be used to identify the
    source input data.
    :var transientFileReadTickets: The transient file tickets. This is for future use.
    :var transientFileTicketInfos: The list of 'TransientFileTicketInfo' objects for retrieving generated report file.
    A 'TransientFileTicketInfo' object contains the file name, and the transient ticket information for the generated
    report file.
    """
    clientID: str = ''
    transientFileReadTickets: List[str] = ()
    transientFileTicketInfos: List[TransientFileTicketInfo] = ()


@dataclass
class GeneratePrintReportsResponse(TcBaseObj):
    """
    A structure that contains list of print report outputs, and error information, if any.
    
    :var outputs: The list of output structure elements.
    :var serviceData: Service data including partial errors that are mapped to the client id from the input.
    :var executedAsynchronously: True value indicates that the operation was executed asynchronously.
    """
    outputs: List[GeneratePrintReportsOutput] = ()
    serviceData: ServiceData = None
    executedAsynchronously: bool = False


@dataclass
class GenerateReportsCriteria2Async(TcBaseObj):
    """
    Data needed to generate a report.
    
    :var clientId: The client identifier for routing purposes.
    :var reportDefinition: The ReportDefinition object.
    :var datasetCtxObj: The context Dataset where generated report Dataset will be attached to.
    :var relationName: The relation name to be used to attach generated report Dataset to context Dataset.
    :var datasetType: The Dataset type to be used to store the report.
    :var reportOptionsNames: A list of option names in a series of Name/Value pairs used to specify additional criteria
    (optional).
    :var reportOptionsValues: A list of option values in a series of Name/Value pairs used to specify additional
    criteria (optional).
    :var reportName: The name of the report if reportDefinition is null.
    :var stylesheet: The report style sheet. The stylesheet is selectable from a list of already defined stylesheets on
    the ReportDefinition object. It is typically an XSL (saved in the DB as a Dataset )(optional).
    :var stylesheetName: The name of the report style if stylesheet is not supplied. If neither stylesheet nor
    stylesheetName is specified, a raw xml file is generated as report.
    :var contextObjects: A list of context object(s) (required for item reports).
    :var datasetName: The name of containing DataSet (optional).If supplied, the generated report will be saved as
    Dataset in Teamcenter,otherwise no dataset created for the report.
    :var criteriaNames: A list of strings containing the Names in a series of Name/Value pairs used to specify criteria
    for saved queries(optional).
    :var criteriaValues: A list of strings containing the Values in a series of Name/Value pairs used to specify
    criteria for saved queries(optional).
    :var datasetCtxUID: The uid for the context Dataset if datasetCtxObj is not supplied.
    """
    clientId: str = ''
    reportDefinition: ReportDefinition = None
    datasetCtxObj: BusinessObject = None
    relationName: str = ''
    datasetType: str = ''
    reportOptionsNames: List[str] = ()
    reportOptionsValues: List[str] = ()
    reportName: str = ''
    stylesheet: BusinessObject = None
    stylesheetName: str = ''
    contextObjects: List[BusinessObject] = ()
    datasetName: str = ''
    criteriaNames: List[str] = ()
    criteriaValues: List[str] = ()
    datasetCtxUID: str = ''


@dataclass
class GenerateReportsResponse2(TcBaseObj):
    """
    Contains file ticket of generated report and synchronize flag as well as service data.
    
    :var transientFileReadTickets: File ticket returned to client to download and the file stored in FMS via this file
    read ticket
    
    :var transientFileTicketInfos: A list of TransientFileTicketInfo object(s) for retrieving generated report file.
    The TransientFileTicketInfo object includes the file name, ticket information.
    :var serviceData: The service data.
    :var asyncFlagInfo: Asynchronous flag,if true  pop up notification on client will be displayed.
    """
    transientFileReadTickets: List[str] = ()
    transientFileTicketInfos: List[TransientFileTicketInfo] = ()
    serviceData: ServiceData = None
    asyncFlagInfo: bool = False


@dataclass
class PrintReportsCriteria(TcBaseObj):
    """
    This structure contains the input details for the generation of the print report.
    
    :var clientID: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var inputObject: The input object for which the print report is to be generated.
    :var isBatchPrint: True value indicates to also send the report to physical printer for printing.
    :var printInfos: The list of structures containing print configuration info. This info will be used only when the
    'isBatchPrint' is set to true.
    """
    clientID: str = ''
    inputObject: BusinessObject = None
    isBatchPrint: bool = False
    printInfos: List[PrintSubmitRequestInfo] = ()


@dataclass
class PrintReportsCriteriaAsync(TcBaseObj):
    """
    This structure contains the input details for the generation of the print report.
    
    :var clientID: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var inputObject: The input object for which the print report is to be generated.
    :var isBatchPrint: True value indicates to also send the generated report to physical printer for printing.
    :var printInfos: The list of structures containing print configuration info. This info will be used only when the
    'isBatchPrint' is set to true.
    """
    clientID: str = ''
    inputObject: BusinessObject = None
    isBatchPrint: bool = False
    printInfos: List[PrintSubmitRequestInfo] = ()


@dataclass
class PrintSubmitRequestInfo(TcBaseObj):
    """
    The structure containing the info needed to submit the print request.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var printObjs: A list of Teamcenter business objects (ItemRevision, Item, Dataset) that will be printed.
    :var collate: When two or more copies are printed, this specifies whether the printed pages are collated.
    :var printToScale: The scaling factor. Specifies the scaling factor from 0.000001 to 100.0, applied to an image
    when its printed.
    :var orientation: The paper orientation of "best fit", "portrait" or "landscape".
    :var bannerPage: Specifies whether to print a page including the defined stamps and listing additional data as
    specified by the VVCP setup(i.e. the Lifecycle Visualization print configuration) The following values can be
    specified: "Off", "Single", or "All Files".
    :var extraInfo: The placeholder for extra name value pair (string, string) information.
    :var printerConfigurationName: The PrintConfiguration object name.
    :var printerName: The printer name.
    :var colorMode: The print color mode. The possible values are "Color" or "Monochrome".
    :var userStamp: Text for a user stamp to be applied in addition to any existing system stamp configuration.
    :var paperSize: The print paper size. The sample values are "Legal", "Letter", "11x17" etc.
    :var printStamp: Where the print stamp is applied: "first page", "the banner page", or "all pages".
    :var pageRange: The range of pages to print. Empty value indicates to print all pages.
    :var numberCopies: The number of copies.
    """
    clientID: str = ''
    printObjs: List[BusinessObject] = ()
    collate: bool = False
    printToScale: str = ''
    orientation: str = ''
    bannerPage: str = ''
    extraInfo: AttributeMap = None
    printerConfigurationName: str = ''
    printerName: str = ''
    colorMode: str = ''
    userStamp: str = ''
    paperSize: str = ''
    printStamp: str = ''
    pageRange: str = ''
    numberCopies: str = ''


"""
A map of attribute name to value pairs.
"""
AttributeMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExternalNodeInfo(TcBaseObj):
    """
    This structure holds BOPLine objects along with their UI location and corresponding BOPLine objects connected via
    scope flows relationship.
    
    :var externalNode: The BOPLine object corresponding to absolute occurrence stored in Mfg0UILocationForm attached to
    the input BOPLine. The type of BOPLine objects can be Mfg0BvrOperation, Mfg0BvrProcess, Mfg0BvrProcessArea, and
    Mfg0BvrWorkarea.
    :var uiLocation: UI location of the BOPLine object. This is MEUILocation property of BOMLine persisted in the form
    attached to the contextLine. It is comma separated list of x-coordinate, y-coordinate, height and width
    representing externalNode position. e.g. "27, 24, 84, 43".
    :var connectedNodes: A list of BOPLine objects connected via scope flow relationship to the BOPLine object whose
    absolute occurrence is stored in the Mfg0UILocationForm attached to the input BOPLine. The type of BOPLine objects
    can be Mfg0BvrOperation, Mfg0BvrProcess, Mfg0BvrProcessArea, and Mfg0BvrWorkarea.
    """
    externalNode: BusinessObject = None
    uiLocation: str = ''
    connectedNodes: List[BusinessObject] = ()


@dataclass
class LABatchDetails(TcBaseObj):
    """
    Schedule timing information deciding when and how many times the operation is to be executed.
    
    :var identifier: Operation identifier string (Valid value: laAsyncResolve)
    :var mode: The mode in which the asynchronous operation is to be executed. Valid values are background, inprocess
    and blocking.
    :var site: The multisite site id on which the asynchronous operation is to be executed.
    :var priority: Priority of the asynchronous operation. Valid values 1(low) -3 (high).
    :var startTime: Start time of the asynchronous operation.
    :var endTime: End time of the asynchronous operation.
    :var daysOfWeek: A list of boolean values which represent each day of the week. The value is set to true for the
    days that you want the operation to be executed. The week starts on Sunday.
    :var endAfterOccurrences: Stop executing the operation after these many executions Valid values are positvie
    integers, a value of 0 indicates no repetition.
    :var primaryObjects: List of Teamcenter objects taking part in the asynchronous operation.  These objects are
    required by the dispatcher service to show in the user interface and also for post operation handling.
    :var secondaryObjects: List of Teamcenter objects taking part in the asynchronous operation.  These objects are
    required by the dispatcher service to show in the user interface and also for post operation handling.
    """
    identifier: str = ''
    mode: str = ''
    site: int = 0
    priority: int = 0
    startTime: datetime = None
    endTime: datetime = None
    daysOfWeek: List[bool] = ()
    endAfterOccurrences: int = 0
    primaryObjects: List[BusinessObject] = ()
    secondaryObjects: List[BusinessObject] = ()


@dataclass
class LAResolveAsyncData(TcBaseObj):
    """
    This data structure contains the scope for which the logical assignments are to be resolved, the products to
    resolve them and also whether to resolve them recursively. All the parameters are mandatory.
    
    :var scopeLines: Process(es) and/or Operation(s) of type Mfg0BvrProcess and Mfg0BvrOperation respectively. These
    lines are from Enterprise Bill of Process (EBOP) stucture for which logical assignments are to be resolved.
    :var products: Products to use for resolving the logical assignments. Top BOMLine(s) of product structure(s).
    :var resolveRecursively: If true, the scopeLines are resolved recursively i.e their children are resolved and so on.
    """
    scopeLines: List[BusinessObject] = ()
    products: List[BusinessObject] = ()
    resolveRecursively: bool = False


@dataclass
class ScopeFlowInfo(TcBaseObj):
    """
    This structure holds the predecessor and the successor BOPLine object of Mfg0BvrScopeFlow object to be deleted.
    
    
    :var predecessor: The predecessor BOPLine of the Mfg0BvrScopeFlow object.
    :var successor: The successor BOPLine of the Mfg0BvrScopeFlow object.
    """
    predecessor: BusinessObject = None
    successor: BusinessObject = None


@dataclass
class UILocationInfo(TcBaseObj):
    """
    This structure holds input BOPLine object and corresponding BOPLine objects & UI locations stored in the form
    attached to it. The type of BOPLine objects can be Mfg0BvrProcess, Mfg0BvrOperation, Mfg0BvrProcessArea, and
    Mfg0BvrWorkarea.
    
    :var contextLine: The input BOPLine object. The type of BOPLine objects can be Mfg0BvrOperation, Mfg0BvrProcess,
    Mfg0BvrProcessArea or Mfg0BvrWorkarea.
    :var externalNodesInfo: A list of BOPLine objects along with their UI location and corresponding BOPLine objects
    connected via scope flows relationship.
    """
    contextLine: BusinessObject = None
    externalNodesInfo: List[ExternalNodeInfo] = ()


@dataclass
class UILocationsInfoResponse(TcBaseObj):
    """
    This structure holds the BOPLine objects and UI locations of these BOPLine objects corresponding to the values of
    absolute occurrence and UI location property in Mfg0UILocationForm attached to the input context BOPLine objects
    along with service data which may contain following partial errors:
    251001 - An internal MFGBVR error has occurred. Please contact your system administrator.
    46110 - Unable to find Parent for 'Proc1'.
    
    
    :var uiLocationsInfo: A list of data containing input BOPLine objects and corresponding BOPLine objects & UI
    locations stored in the form attached to it. The type of BOPLine objects can be Mfg0BvrProcess, Mfg0BvrOperation,
    Mfg0BvrProcessArea, and Mfg0BvrWorkarea.
    :var serviceData: Standard service data.
    """
    uiLocationsInfo: List[UILocationInfo] = ()
    serviceData: ServiceData = None
